@@ -58,6 +58,8 @@ struct _ClutterActorMetaPrivate
   gchar *name;
 
   guint is_enabled : 1;
+
+  gint priority;
 };
 
 enum
@@ -224,9 +226,9 @@ clutter_actor_meta_class_init (ClutterActorMetaClass *klass)
   gobject_class->finalize = clutter_actor_meta_finalize;
   gobject_class->set_property = clutter_actor_meta_set_property;
   gobject_class->get_property = clutter_actor_meta_get_property;
-  _clutter_object_class_install_properties (gobject_class,
-                                            PROP_LAST,
-                                            obj_props);
+  g_object_class_install_properties (gobject_class,
+                                     PROP_LAST,
+                                     obj_props);
 }
 
 void
@@ -237,6 +239,8 @@ clutter_actor_meta_init (ClutterActorMeta *self)
                                             ClutterActorMetaPrivate);
 
   self->priv->is_enabled = TRUE;
+
+  self->priv->priority = CLUTTER_ACTOR_META_PRIORITY_DEFAULT;
 }
 
 /**
@@ -262,7 +266,7 @@ clutter_actor_meta_set_name (ClutterActorMeta *meta,
   g_free (meta->priv->name);
   meta->priv->name = g_strdup (name);
 
-  _clutter_notify_by_pspec (G_OBJECT (meta), obj_props[PROP_NAME]);
+  g_object_notify_by_pspec (G_OBJECT (meta), obj_props[PROP_NAME]);
 }
 
 /**
@@ -308,7 +312,7 @@ clutter_actor_meta_set_enabled (ClutterActorMeta *meta,
 
   meta->priv->is_enabled = is_enabled;
 
-  _clutter_notify_by_pspec (G_OBJECT (meta), obj_props[PROP_ENABLED]);
+  g_object_notify_by_pspec (G_OBJECT (meta), obj_props[PROP_ENABLED]);
 }
 
 /**
@@ -367,6 +371,40 @@ clutter_actor_meta_get_actor (ClutterActorMeta *meta)
   return meta->priv->actor;
 }
 
+void
+_clutter_actor_meta_set_priority (ClutterActorMeta *meta,
+                                  gint priority)
+{
+  g_return_if_fail (CLUTTER_IS_ACTOR_META (meta));
+
+  /* This property shouldn't be modified after the actor meta is in
+     use because ClutterMetaGroup doesn't resort the list when it
+     changes. If we made the priority public then we could either make
+     the priority a construct-only property or listen for
+     notifications on the property from the ClutterMetaGroup and
+     resort. */
+  g_return_if_fail (meta->priv->actor == NULL);
+
+  meta->priv->priority = priority;
+}
+
+gint
+_clutter_actor_meta_get_priority (ClutterActorMeta *meta)
+{
+  g_return_val_if_fail (CLUTTER_IS_ACTOR_META (meta), 0);
+
+  return meta->priv->priority;
+}
+
+gboolean
+_clutter_actor_meta_is_internal (ClutterActorMeta *meta)
+{
+  gint priority = meta->priv->priority;
+
+  return (priority <= CLUTTER_ACTOR_META_PRIORITY_INTERNAL_LOW ||
+          priority >= CLUTTER_ACTOR_META_PRIORITY_INTERNAL_HIGH);
+}
+
 /*
  * ClutterMetaGroup: a collection of ClutterActorMeta instances
  */
@@ -408,6 +446,8 @@ void
 _clutter_meta_group_add_meta (ClutterMetaGroup *group,
                               ClutterActorMeta *meta)
 {
+  GList *prev = NULL, *l;
+
   if (meta->priv->actor != NULL)
     {
       g_warning ("The meta of type '%s' with name '%s' is "
@@ -422,7 +462,22 @@ _clutter_meta_group_add_meta (ClutterMetaGroup *group,
       return;
     }
 
-  group->meta = g_list_append (group->meta, meta);
+  /* Find a meta that has lower priority and insert before that */
+  for (l = group->meta; l; l = l->next)
+    if (_clutter_actor_meta_get_priority (l->data) <
+        _clutter_actor_meta_get_priority (meta))
+      break;
+    else
+      prev = l;
+
+  if (prev == NULL)
+    group->meta = g_list_prepend (group->meta, meta);
+  else
+    {
+      prev->next = g_list_prepend (prev->next, meta);
+      prev->next->prev = prev;
+    }
+
   g_object_ref_sink (meta);
 
   _clutter_actor_meta_set_actor (meta, group->actor);
@@ -473,6 +528,30 @@ _clutter_meta_group_peek_metas (ClutterMetaGroup *group)
   return group->meta;
 }
 
+/*
+ * _clutter_meta_group_get_metas_no_internal:
+ * @group: a #ClutterMetaGroup
+ *
+ * Returns a new allocated list containing all of the metas that don't
+ * have an internal priority.
+ *
+ * Return value: A GList containing non-internal metas. Free with
+ * g_list_free.
+ */
+GList *
+_clutter_meta_group_get_metas_no_internal (ClutterMetaGroup *group)
+{
+  GList *ret = NULL;
+  GList *l;
+
+  /* Build a new list filtering out the internal metas */
+  for (l = group->meta; l; l = l->next)
+    if (!_clutter_actor_meta_is_internal (l->data))
+      ret = g_list_prepend (ret, l->data);
+
+  return g_list_reverse (ret);
+}
+
 /*
  * _clutter_meta_group_clear_metas:
  * @group: a #ClutterMetaGroup
@@ -490,6 +569,42 @@ _clutter_meta_group_clear_metas (ClutterMetaGroup *group)
   group->meta = NULL;
 }
 
+/*
+ * _clutter_meta_group_clear_metas_no_internal:
+ * @group: a #ClutterMetaGroup
+ *
+ * Clears @group of all #ClutterActorMeta instances that don't have an
+ * internal priority and releases the reference on them
+ */
+void
+_clutter_meta_group_clear_metas_no_internal (ClutterMetaGroup *group)
+{
+  GList *internal_list = NULL;
+  GList *l, *next;
+
+  for (l = group->meta; l; l = next)
+    {
+      next = l->next;
+
+      if (_clutter_actor_meta_is_internal (l->data))
+        {
+          if (internal_list)
+            internal_list->prev = l;
+          l->next = internal_list;
+          l->prev = NULL;
+          internal_list = l;
+        }
+      else
+        {
+          _clutter_actor_meta_set_actor (l->data, NULL);
+          g_object_unref (l->data);
+          g_list_free_1 (l);
+        }
+    }
+
+  group->meta = g_list_reverse (internal_list);
+}
+
 /*
  * _clutter_meta_group_get_meta:
  * @group: a #ClutterMetaGroup
@@ -515,3 +630,20 @@ _clutter_meta_group_get_meta (ClutterMetaGroup *group,
 
   return NULL;
 }
+
+/*< private >
+ * clutter_actor_meta_get_debug_name:
+ * @meta: a #ClutterActorMeta
+ *
+ * Retrieves the name of the @meta for debugging purposes.
+ *
+ * Return value: (transfer none): the name of the @meta. The returned
+ *   string is owned by the @meta instance and it should not be
+ *   modified or freed
+ */
+const gchar *
+_clutter_actor_meta_get_debug_name (ClutterActorMeta *meta)
+{
+  return meta->priv->name != NULL ? meta->priv->name
+                                  : G_OBJECT_TYPE_NAME (meta);
+}
