@@ -42,67 +42,37 @@
 #include "config.h"
 #endif
 
-#define CLUTTER_ENABLE_EXPERIMENTAL_API
-
 #include "clutter-backend-private.h"
 #include "clutter-debug.h"
 #include "clutter-event-private.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
+#include "clutter-profile.h"
 #include "clutter-stage-manager-private.h"
 #include "clutter-stage-private.h"
-#include "clutter-stage-window.h"
 #include "clutter-version.h"
 
-#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
-#include "deprecated/clutter-backend.h"
-
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-#include "wayland/clutter-wayland-compositor.h"
-#endif
-
 #include <cogl/cogl.h>
 
-#ifdef CLUTTER_INPUT_X11
-#include "x11/clutter-backend-x11.h"
-#endif
-#ifdef CLUTTER_INPUT_WIN32
-#include "win32/clutter-backend-win32.h"
-#endif
-#ifdef CLUTTER_INPUT_OSX
-#include "osx/clutter-backend-osx.h"
-#endif
-#ifdef CLUTTER_INPUT_GDK
-#include "gdk/clutter-backend-gdk.h"
-#endif
-#ifdef CLUTTER_INPUT_EVDEV
-#include "evdev/clutter-device-manager-evdev.h"
-#endif
-#ifdef CLUTTER_INPUT_TSLIB
-/* XXX - should probably warn, here */
-#include "tslib/clutter-event-tslib.h"
-#endif
-#ifdef CLUTTER_WINDOWING_EGL
-#include "egl/clutter-backend-eglnative.h"
-#endif
-#ifdef CLUTTER_INPUT_WAYLAND
-#include "wayland/clutter-device-manager-wayland.h"
-#endif
-#ifdef CLUTTER_WINDOWING_MIR
-#include "mir/clutter-backend-mir.h"
-#endif
-#ifdef CLUTTER_INPUT_MIR
-#include "mir/clutter-device-manager-mir.h"
-#endif
-
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-#include <cogl/cogl-wayland-server.h>
-#include <wayland-server.h>
-#include "wayland/clutter-wayland-compositor.h"
-#endif
+G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
 
 #define DEFAULT_FONT_NAME       "Sans 10"
 
+#define CLUTTER_BACKEND_GET_PRIVATE(obj) \
+(G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_BACKEND, ClutterBackendPrivate))
+
+struct _ClutterBackendPrivate
+{
+  cairo_font_options_t *font_options;
+
+  gchar *font_name;
+
+  gfloat units_per_em;
+  gint32 units_serial;
+
+  GList *event_translators;
+};
+
 enum
 {
   RESOLUTION_CHANGED,
@@ -112,30 +82,22 @@ enum
   LAST_SIGNAL
 };
 
-G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT)
-
 static guint backend_signals[LAST_SIGNAL] = { 0, };
 
-/* Global for being able to specify a compositor side wayland display
- * pointer before clutter initialization */
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-static struct wl_display *_wayland_compositor_display;
-#endif
-
-static const char *allowed_backend;
-
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
+  ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
 
   /* clear the events still in the queue of the main context */
   _clutter_clear_events_queue ();
 
   /* remove all event translators */
-  g_clear_pointer (&backend->event_translators, g_list_free);
-
-  g_clear_pointer (&backend->dummy_onscreen, cogl_object_unref);
+  if (priv->event_translators != NULL)
+    {
+      g_list_free (priv->event_translators);
+      priv->event_translators = NULL;
+    }
 
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
@@ -145,9 +107,7 @@ clutter_backend_finalize (GObject *gobject)
 {
   ClutterBackend *backend = CLUTTER_BACKEND (gobject);
 
-  g_source_destroy (backend->cogl_source);
-
-  g_free (backend->font_name);
+  g_free (backend->priv->font_name);
   clutter_backend_set_font_options (backend, NULL);
 
   G_OBJECT_CLASS (clutter_backend_parent_class)->finalize (gobject);
@@ -195,10 +155,14 @@ get_units_per_em (ClutterBackend       *backend,
       if (is_absolute)
         font_size = (gdouble) pango_size / PANGO_SCALE;
       else
-        font_size = dpi * ((gdouble) pango_size / PANGO_SCALE) / 72.0f;
+        font_size = (gdouble) pango_size / PANGO_SCALE
+                  * dpi
+                  / 96.0f;
 
       /* 10 points at 96 DPI is 13.3 pixels */
-      units_per_em = (1.2f * font_size) * dpi / 96.0f;
+      units_per_em = (1.2f * font_size)
+                   * dpi
+                   / 96.0f;
     }
   else
     units_per_em = -1.0f;
@@ -212,447 +176,49 @@ get_units_per_em (ClutterBackend       *backend,
 static void
 clutter_backend_real_resolution_changed (ClutterBackend *backend)
 {
+  ClutterBackendPrivate *priv = backend->priv;
   ClutterMainContext *context;
   ClutterSettings *settings;
-  gdouble resolution;
   gint dpi;
 
   settings = clutter_settings_get_default ();
   g_object_get (settings, "font-dpi", &dpi, NULL);
 
-  if (dpi < 0)
-    resolution = 96.0;
-  else
-    resolution = dpi / 1024.0;
-
   context = _clutter_context_get_default ();
   if (context->font_map != NULL)
-    cogl_pango_font_map_set_resolution (context->font_map, resolution);
+    cogl_pango_font_map_set_resolution (context->font_map, dpi / 1024.0);
 
-  backend->units_per_em = get_units_per_em (backend, NULL);
-  backend->units_serial += 1;
+  priv->units_per_em = get_units_per_em (backend, NULL);
+  priv->units_serial += 1;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->units_per_em);
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
 clutter_backend_real_font_changed (ClutterBackend *backend)
 {
-  backend->units_per_em = get_units_per_em (backend, NULL);
-  backend->units_serial += 1;
+  ClutterBackendPrivate *priv = backend->priv;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->units_per_em);
-}
+  priv->units_per_em = get_units_per_em (backend, NULL);
+  priv->units_serial += 1;
 
-static gboolean
-clutter_backend_do_real_create_context (ClutterBackend  *backend,
-                                        CoglDriver       driver_id,
-                                        GError         **error)
-{
-  ClutterBackendClass *klass;
-  CoglSwapChain *swap_chain;
-  GError *internal_error;
-
-  if (backend->cogl_context != NULL)
-    return TRUE;
-
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-
-  swap_chain = NULL;
-  internal_error = NULL;
-
-  CLUTTER_NOTE (BACKEND, "Creating Cogl renderer");
-  if (klass->get_renderer != NULL)
-    backend->cogl_renderer = klass->get_renderer (backend, &internal_error);
-  else
-    backend->cogl_renderer = cogl_renderer_new ();
-
-  if (backend->cogl_renderer == NULL)
-    goto error;
-
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-  /* If the application is trying to act as a Wayland compositor then
-     it needs to have an EGL-based renderer backend */
-  if (_wayland_compositor_display)
-    cogl_renderer_add_constraint (backend->cogl_renderer,
-                                  COGL_RENDERER_CONSTRAINT_USES_EGL);
-#endif
-
-  CLUTTER_NOTE (BACKEND, "Connecting the renderer");
-  cogl_renderer_set_driver (backend->cogl_renderer, driver_id);
-  if (!cogl_renderer_connect (backend->cogl_renderer, &internal_error))
-    goto error;
-
-  CLUTTER_NOTE (BACKEND, "Creating Cogl swap chain");
-  swap_chain = cogl_swap_chain_new ();
-
-  CLUTTER_NOTE (BACKEND, "Creating Cogl display");
-  if (klass->get_display != NULL)
-    {
-      backend->cogl_display = klass->get_display (backend,
-                                                  backend->cogl_renderer,
-                                                  swap_chain,
-                                                  &internal_error);
-    }
-  else
-    {
-      CoglOnscreenTemplate *tmpl;
-      gboolean res;
-
-      tmpl = cogl_onscreen_template_new (swap_chain);
-
-      /* XXX: I have some doubts that this is a good design.
-       *
-       * Conceptually should we be able to check an onscreen_template
-       * without more details about the CoglDisplay configuration?
-       */
-      res = cogl_renderer_check_onscreen_template (backend->cogl_renderer,
-                                                   tmpl,
-                                                   &internal_error);
-
-      if (!res)
-        goto error;
-
-      backend->cogl_display = cogl_display_new (backend->cogl_renderer, tmpl);
-
-      /* the display owns the template */
-      cogl_object_unref (tmpl);
-    }
-
-  if (backend->cogl_display == NULL)
-    goto error;
-
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-  cogl_wayland_display_set_compositor_display (backend->cogl_display,
-                                               _wayland_compositor_display);
-#endif
-
-  CLUTTER_NOTE (BACKEND, "Setting up the display");
-  if (!cogl_display_setup (backend->cogl_display, &internal_error))
-    goto error;
-
-  CLUTTER_NOTE (BACKEND, "Creating the Cogl context");
-  backend->cogl_context = cogl_context_new (backend->cogl_display, &internal_error);
-  if (backend->cogl_context == NULL)
-    goto error;
-
-  /* the display owns the renderer and the swap chain */
-  cogl_object_unref (backend->cogl_renderer);
-  cogl_object_unref (swap_chain);
-
-  return TRUE;
-
-error:
-  if (backend->cogl_display != NULL)
-    {
-      cogl_object_unref (backend->cogl_display);
-      backend->cogl_display = NULL;
-    }
-
-  if (backend->cogl_renderer != NULL)
-    {
-      cogl_object_unref (backend->cogl_renderer);
-      backend->cogl_renderer = NULL;
-    }
-
-  if (swap_chain != NULL)
-    cogl_object_unref (swap_chain);
-
-  return FALSE;
-}
-
-static gboolean
-clutter_backend_real_create_context (ClutterBackend  *backend,
-                                     GError         **error)
-{
-  static const struct {
-    const char *driver_name;
-    CoglDriver driver_id;
-  } known_drivers[] = {
-    { "GL3", COGL_DRIVER_GL3 },
-    { "GL (Legacy)", COGL_DRIVER_GL },
-    { "GLES 2.0", COGL_DRIVER_GLES2 },
-    { "ANY", COGL_DRIVER_ANY },
-  };
-
-  GError *internal_error = NULL;
-  int i;
-
-  for (i = 0; i < G_N_ELEMENTS (known_drivers); i++)
-    {
-      CLUTTER_NOTE (BACKEND, "Checking for the %s driver", known_drivers[i].driver_name);
-
-      if (clutter_backend_do_real_create_context (backend, known_drivers[i].driver_id, &internal_error))
-        break;
-
-      if (internal_error)
-        {
-          CLUTTER_NOTE (BACKEND, "Unable to use the %s driver: %s",
-                        known_drivers[i].driver_name,
-                        internal_error->message);
-          g_clear_error (&internal_error);
-        }
-    }
-
-  if (backend->cogl_context == NULL)
-    {
-      if (internal_error != NULL)
-        g_propagate_error (error, internal_error);
-      else
-        g_set_error_literal (error, CLUTTER_INIT_ERROR,
-                             CLUTTER_INIT_ERROR_BACKEND,
-                            _("Unable to initialize the Clutter backend"));
-
-      return FALSE;
-    }
-
-  backend->cogl_source = cogl_glib_source_new (backend->cogl_context,
-                                               G_PRIORITY_DEFAULT);
-  g_source_attach (backend->cogl_source, NULL);
-
-  return TRUE;
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
-clutter_backend_real_ensure_context (ClutterBackend *backend,
-                                     ClutterStage   *stage)
+clutter_backend_real_redraw (ClutterBackend *backend,
+                             ClutterStage   *stage)
 {
-  ClutterStageWindow *stage_impl;
-  CoglFramebuffer *framebuffer;
+  ClutterStageWindow *impl;
 
-  if (stage == NULL)
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
     return;
 
-  stage_impl = _clutter_stage_get_window (stage);
-  if (stage_impl == NULL)
+  impl = _clutter_stage_get_window (stage);
+  if (impl == NULL)
     return;
 
-  framebuffer = _clutter_stage_window_get_active_framebuffer (stage_impl);
-  if (framebuffer == NULL)
-    return;
-
-  cogl_set_framebuffer (framebuffer);
-}
-
-static ClutterFeatureFlags
-clutter_backend_real_get_features (ClutterBackend *backend)
-{
-  ClutterFeatureFlags flags = 0;
-
-  if (cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_MULTIPLE_ONSCREEN))
-    {
-      CLUTTER_NOTE (BACKEND, "Cogl supports multiple onscreen framebuffers");
-      flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
-    }
-  else
-    {
-      CLUTTER_NOTE (BACKEND, "Cogl only supports one onscreen framebuffer");
-      flags |= CLUTTER_FEATURE_STAGE_STATIC;
-    }
-
-  if (cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_THROTTLE))
-    {
-      CLUTTER_NOTE (BACKEND, "Cogl supports swap buffers throttling");
-      flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
-    }
-  else
-    CLUTTER_NOTE (BACKEND, "Cogl doesn't support swap buffers throttling");
-
-  if (cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_BUFFERS_EVENT))
-    {
-      CLUTTER_NOTE (BACKEND, "Cogl supports swap buffers complete events");
-      flags |= CLUTTER_FEATURE_SWAP_EVENTS;
-    }
-
-  return flags;
-}
-
-static ClutterStageWindow *
-clutter_backend_real_create_stage (ClutterBackend  *backend,
-                                   ClutterStage    *wrapper,
-                                   GError         **error)
-{
-  ClutterBackendClass *klass;
-
-  if (!clutter_feature_available (CLUTTER_FEATURE_STAGE_MULTIPLE))
-    {
-      ClutterStageManager *manager = clutter_stage_manager_get_default ();
-
-      if (clutter_stage_manager_get_default_stage (manager) != NULL)
-        {
-          g_set_error (error, CLUTTER_INIT_ERROR,
-                       CLUTTER_INIT_ERROR_BACKEND,
-                       _("The backend of type '%s' does not support "
-                         "creating multiple stages"),
-                       G_OBJECT_TYPE_NAME (backend));
-          return NULL;
-        }
-    }
-
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  g_assert (klass->stage_window_type != G_TYPE_INVALID);
-
-  return g_object_new (klass->stage_window_type,
-                       "backend", backend,
-                       "wrapper", wrapper,
-                       NULL);
-}
-
-ClutterBackend *
-_clutter_create_backend (void)
-{
-  const char *backend = allowed_backend;
-  ClutterBackend *retval = NULL;
-
-  if (backend == NULL)
-    {
-      const char *backend_env = g_getenv ("CLUTTER_BACKEND");
-
-      if (backend_env != NULL)
-	backend = g_intern_string (backend_env);
-    }
-
-#ifdef CLUTTER_WINDOWING_OSX
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_OSX))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_OSX, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_WIN32
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_WIN32))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_WIN32, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_GDK
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_GDK))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_GDK, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_X11
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_X11))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_X11, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_WAYLAND
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_WAYLAND))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_WAYLAND, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_EGL
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_EGL))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_EGL_NATIVE, NULL);
-  else
-#endif
-#ifdef CLUTTER_WINDOWING_MIR
-  if (backend == NULL || backend == I_(CLUTTER_WINDOWING_MIR))
-    retval = g_object_new (CLUTTER_TYPE_BACKEND_MIR, NULL);
-  else
-#endif
-  if (backend == NULL)
-    g_error ("No default Clutter backend found.");
-  else
-    g_error ("Unsupported Clutter backend: '%s'", backend);
-
-  return retval;
-}
-
-static void
-clutter_backend_real_init_events (ClutterBackend *backend)
-{
-  const char *input_backend = NULL;
-
-  input_backend = g_getenv ("CLUTTER_INPUT_BACKEND");
-  if (input_backend != NULL)
-    input_backend = g_intern_string (input_backend);
-
-#ifdef CLUTTER_INPUT_OSX
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_OSX) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_OSX)))
-    {
-      _clutter_backend_osx_events_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_WIN32
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WIN32) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_WIN32)))
-    {
-      _clutter_backend_win32_events_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_X11
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_X11)))
-    {
-      _clutter_backend_x11_events_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_GDK
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_GDK) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_GDK)))
-    {
-      _clutter_backend_gdk_events_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_EVDEV
-  /* Evdev can be used regardless of the windowing system */
-  if ((input_backend != NULL && strcmp (input_backend, CLUTTER_INPUT_EVDEV) == 0)
-#ifdef CLUTTER_WINDOWING_EGL
-      /* but we do want to always use it for EGL native */
-      || clutter_check_windowing_backend (CLUTTER_WINDOWING_EGL)
-#endif
-      )
-    {
-      _clutter_events_evdev_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_TSLIB
-  /* Tslib can be used regardless of the windowing system */
-  if (input_backend != NULL &&
-      strcmp (input_backend, CLUTTER_INPUT_TSLIB) == 0)
-    {
-      _clutter_events_tslib_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_WAYLAND
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WAYLAND) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_WAYLAND)))
-    {
-      _clutter_events_wayland_init (backend);
-    }
-  else
-#endif
-#ifdef CLUTTER_INPUT_MIR
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_MIR) &&
-      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_MIR)))
-    {
-      _clutter_events_mir_init (backend);
-    }
-  else
-#endif
-  if (input_backend != NULL)
-    {
-      if (input_backend != I_(CLUTTER_INPUT_NULL))
-        g_error ("Unrecognized input backend '%s'", input_backend);
-    }
-  else
-    g_error ("Unknown input backend");
-}
-
-static ClutterDeviceManager *
-clutter_backend_real_get_device_manager (ClutterBackend *backend)
-{
-  if (G_UNLIKELY (backend->device_manager == NULL))
-    {
-      g_critical ("No device manager available, expect broken input");
-      return NULL;
-    }
-
-  return backend->device_manager;
+  _clutter_stage_window_redraw (impl);
 }
 
 static gboolean
@@ -660,9 +226,10 @@ clutter_backend_real_translate_event (ClutterBackend *backend,
                                       gpointer        native,
                                       ClutterEvent   *event)
 {
+  ClutterBackendPrivate *priv = backend->priv;
   GList *l;
 
-  for (l = backend->event_translators;
+  for (l = priv->event_translators;
        l != NULL;
        l = l->next)
     {
@@ -691,7 +258,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   gobject_class->dispose = clutter_backend_dispose;
   gobject_class->finalize = clutter_backend_finalize;
 
-  klass->stage_window_type = G_TYPE_INVALID;
+  g_type_class_add_private (gobject_class, sizeof (ClutterBackendPrivate));
 
   /**
    * ClutterBackend::resolution-changed:
@@ -749,23 +316,19 @@ clutter_backend_class_init (ClutterBackendClass *klass)
 
   klass->resolution_changed = clutter_backend_real_resolution_changed;
   klass->font_changed = clutter_backend_real_font_changed;
-
-  klass->init_events = clutter_backend_real_init_events;
-  klass->get_device_manager = clutter_backend_real_get_device_manager;
   klass->translate_event = clutter_backend_real_translate_event;
-  klass->create_context = clutter_backend_real_create_context;
-  klass->ensure_context = clutter_backend_real_ensure_context;
-  klass->get_features = clutter_backend_real_get_features;
-  klass->create_stage = clutter_backend_real_create_stage;
+  klass->redraw = clutter_backend_real_redraw;
 }
 
 static void
-clutter_backend_init (ClutterBackend *self)
+clutter_backend_init (ClutterBackend *backend)
 {
-  self->units_per_em = -1.0;
-  self->units_serial = 1;
+  ClutterBackendPrivate *priv;
 
-  self->dummy_onscreen = COGL_INVALID_HANDLE;
+  priv = backend->priv = CLUTTER_BACKEND_GET_PRIVATE (backend);
+
+  priv->units_per_em = -1.0;
+  priv->units_serial = 1;
 }
 
 void
@@ -817,11 +380,14 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
                                GError         **error)
 {
   ClutterBackendClass *klass;
+  ClutterStageManager *stage_manager;
   ClutterStageWindow *stage_window;
 
   g_assert (CLUTTER_IS_BACKEND (backend));
   g_assert (CLUTTER_IS_STAGE (wrapper));
 
+  stage_manager = clutter_stage_manager_get_default ();
+
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->create_stage != NULL)
     stage_window = klass->create_stage (backend, wrapper, error);
@@ -832,10 +398,34 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
     return NULL;
 
   g_assert (CLUTTER_IS_STAGE_WINDOW (stage_window));
+  _clutter_stage_set_window (wrapper, stage_window);
+  _clutter_stage_manager_add_stage (stage_manager, wrapper);
 
   return stage_window;
 }
 
+void
+_clutter_backend_redraw (ClutterBackend *backend,
+                         ClutterStage   *stage)
+{
+  CLUTTER_STATIC_COUNTER (redraw_counter,
+                          "_clutter_backend_redraw counter",
+                          "Increments for each _clutter_backend_redraw call",
+                          0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (redraw_timer,
+                        "Master Clock", /* parent */
+                        "Redrawing",
+                        "The time spent redrawing everything",
+                        0 /* no application private data */);
+
+  CLUTTER_COUNTER_INC (_clutter_uprof_context, redraw_counter);
+  CLUTTER_TIMER_START (_clutter_uprof_context, redraw_timer);
+
+  CLUTTER_BACKEND_GET_CLASS (backend)->redraw (backend, stage);
+
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, redraw_timer);
+}
+
 gboolean
 _clutter_backend_create_context (ClutterBackend  *backend,
                                  GError         **error)
@@ -843,8 +433,10 @@ _clutter_backend_create_context (ClutterBackend  *backend,
   ClutterBackendClass *klass;
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->create_context)
+    return klass->create_context (backend, error);
 
-  return klass->create_context (backend, error);
+  return TRUE;
 }
 
 void
@@ -865,16 +457,15 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
   g_assert (CLUTTER_IS_BACKEND (backend));
   g_assert (CLUTTER_IS_STAGE (stage));
 
-  if (current_context_stage != stage ||
-      !clutter_actor_is_realized (CLUTTER_ACTOR (stage)))
+  if (current_context_stage != stage || !CLUTTER_ACTOR_IS_REALIZED (stage))
     {
       ClutterStage *new_stage = NULL;
 
-      if (!clutter_actor_is_realized (CLUTTER_ACTOR (stage)))
+      if (!CLUTTER_ACTOR_IS_REALIZED (stage))
         {
           new_stage = NULL;
 
-          CLUTTER_NOTE (BACKEND,
+          CLUTTER_NOTE (MULTISTAGE,
                         "Stage [%p] is not realized, unsetting the stage",
                         stage);
         }
@@ -882,23 +473,23 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
         {
           new_stage = stage;
 
-          CLUTTER_NOTE (BACKEND,
+          CLUTTER_NOTE (MULTISTAGE,
                         "Setting the new stage [%p]",
                         new_stage);
         }
 
+      _clutter_backend_ensure_context_internal (backend, new_stage);
+
       /* XXX: Until Cogl becomes fully responsible for backend windows
        * Clutter need to manually keep it informed of the current window size
        *
        * NB: This must be done after we ensure_context above because Cogl
        * always assumes there is a current GL context.
        */
-      if (new_stage != NULL)
+      if (new_stage)
         {
           float width, height;
 
-          _clutter_backend_ensure_context_internal (backend, new_stage);
-
           clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
 
           cogl_onscreen_clutter_backend_set_size (width, height);
@@ -910,8 +501,7 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
            * switch between stages.
            *
            * This dirty mechanism will ensure they are asserted before
-           * the next paint...
-           */
+           * the next paint... */
           _clutter_stage_dirty_viewport (stage);
           _clutter_stage_dirty_projection (stage);
         }
@@ -919,12 +509,12 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
       /* FIXME: With a NULL stage and thus no active context it may make more
        * sense to clean the context but then re call with the default stage 
        * so at least there is some kind of context in place (as to avoid
-       * potential issue of GL calls with no context).
+       * potential issue of GL calls with no context)
        */
       current_context_stage = new_stage;
     }
   else
-    CLUTTER_NOTE (BACKEND, "Stage is the same");
+    CLUTTER_NOTE (MULTISTAGE, "Stage is the same");
 }
 
 
@@ -976,21 +566,26 @@ _clutter_backend_init_events (ClutterBackend *backend)
   g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  klass->init_events (backend);
+  if (klass->init_events)
+    klass->init_events (backend);
 }
 
 gfloat
 _clutter_backend_get_units_per_em (ClutterBackend       *backend,
                                    PangoFontDescription *font_desc)
 {
+  ClutterBackendPrivate *priv;
+
+  priv = backend->priv;
+
   /* recompute for the font description, but do not cache the result */
   if (font_desc != NULL)
     return get_units_per_em (backend, font_desc);
 
-  if (backend->units_per_em < 0)
-    backend->units_per_em = get_units_per_em (backend, NULL);
+  if (priv->units_per_em < 0)
+    priv->units_per_em = get_units_per_em (backend, NULL);
 
-  return backend->units_per_em;
+  return priv->units_per_em;
 }
 
 void
@@ -1141,7 +736,7 @@ clutter_backend_get_double_click_distance (ClutterBackend *backend)
  *
  * Since: 0.4
  *
- * Deprecated: 1.4: Use #ClutterSettings:font-dpi instead
+ * Deprecated: Use #ClutterSettings:font-dpi instead
  */
 void
 clutter_backend_set_resolution (ClutterBackend *backend,
@@ -1192,9 +787,6 @@ clutter_backend_get_resolution (ClutterBackend *backend)
   settings = clutter_settings_get_default ();
   g_object_get (settings, "font-dpi", &resolution, NULL);
 
-  if (resolution < 0)
-    return 96.0;
-
   return resolution / 1024.0;
 }
 
@@ -1219,17 +811,21 @@ void
 clutter_backend_set_font_options (ClutterBackend             *backend,
                                   const cairo_font_options_t *options)
 {
+  ClutterBackendPrivate *priv;
+
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
 
-  if (backend->font_options != options)
+  priv = backend->priv;
+
+  if (priv->font_options != options)
     {
-      if (backend->font_options)
-        cairo_font_options_destroy (backend->font_options);
+      if (priv->font_options)
+        cairo_font_options_destroy (priv->font_options);
 
       if (options)
-        backend->font_options = cairo_font_options_copy (options);
+        priv->font_options = cairo_font_options_copy (options);
       else
-        backend->font_options = NULL;
+        priv->font_options = NULL;
 
       g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
     }
@@ -1250,20 +846,27 @@ clutter_backend_set_font_options (ClutterBackend             *backend,
 const cairo_font_options_t *
 clutter_backend_get_font_options (ClutterBackend *backend)
 {
+  ClutterBackendPrivate *priv;
+
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
 
-  if (G_LIKELY (backend->font_options))
-    return backend->font_options;
+  priv = backend->priv;
 
-  backend->font_options = cairo_font_options_create ();
+  if (G_LIKELY (priv->font_options))
+    return priv->font_options;
 
-  cairo_font_options_set_hint_style (backend->font_options, CAIRO_HINT_STYLE_NONE);
-  cairo_font_options_set_subpixel_order (backend->font_options, CAIRO_SUBPIXEL_ORDER_DEFAULT);
-  cairo_font_options_set_antialias (backend->font_options, CAIRO_ANTIALIAS_DEFAULT);
+  priv->font_options = cairo_font_options_create ();
+
+  cairo_font_options_set_hint_style (priv->font_options,
+                                     CAIRO_HINT_STYLE_NONE);
+  cairo_font_options_set_subpixel_order (priv->font_options,
+                                         CAIRO_SUBPIXEL_ORDER_DEFAULT);
+  cairo_font_options_set_antialias (priv->font_options,
+                                    CAIRO_ANTIALIAS_DEFAULT);
 
   g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
 
-  return backend->font_options;
+  return priv->font_options;
 }
 
 /**
@@ -1306,25 +909,28 @@ clutter_backend_set_font_name (ClutterBackend *backend,
 const gchar *
 clutter_backend_get_font_name (ClutterBackend *backend)
 {
+  ClutterBackendPrivate *priv;
   ClutterSettings *settings;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
 
+  priv = backend->priv;
+
   settings = clutter_settings_get_default ();
 
   /* XXX yuck. but we return a const pointer, so we need to
    * store it in the backend
    */
-  g_free (backend->font_name);
-  g_object_get (settings, "font-name", &backend->font_name, NULL);
+  g_free (priv->font_name);
+  g_object_get (settings, "font-name", &priv->font_name, NULL);
 
-  return backend->font_name;
+  return priv->font_name;
 }
 
 gint32
 _clutter_backend_get_units_serial (ClutterBackend *backend)
 {
-  return backend->units_serial;
+  return backend->priv->units_serial;
 }
 
 gboolean
@@ -1341,122 +947,24 @@ void
 _clutter_backend_add_event_translator (ClutterBackend         *backend,
                                        ClutterEventTranslator *translator)
 {
-  if (g_list_find (backend->event_translators, translator) != NULL)
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) != NULL)
     return;
 
-  backend->event_translators =
-    g_list_prepend (backend->event_translators, translator);
+  priv->event_translators =
+    g_list_prepend (priv->event_translators, translator);
 }
 
 void
 _clutter_backend_remove_event_translator (ClutterBackend         *backend,
                                           ClutterEventTranslator *translator)
 {
-  if (g_list_find (backend->event_translators, translator) == NULL)
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) == NULL)
     return;
 
-  backend->event_translators =
-    g_list_remove (backend->event_translators, translator);
-}
-
-/**
- * clutter_backend_get_cogl_context:
- * @backend: a #ClutterBackend
- *
- * Retrieves the #CoglContext associated with the given clutter
- * @backend. A #CoglContext is required when using some of the
- * experimental 2.0 Cogl API.
- *
- * Since CoglContext is itself experimental API this API should
- * be considered experimental too.
- *
- * This API is not yet supported on OSX because OSX still
- * uses the stub Cogl winsys and the Clutter backend doesn't
- * explicitly create a CoglContext.
- *
- * Return value: (transfer none): The #CoglContext associated with @backend.
- *
- * Since: 1.8
- * Stability: unstable
- */
-CoglContext *
-clutter_backend_get_cogl_context (ClutterBackend *backend)
-{
-  return backend->cogl_context;
-}
-
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-/**
- * clutter_wayland_set_compositor_display:
- * @display: A compositor side struct wl_display pointer
- *
- * This informs Clutter of your compositor side Wayland display
- * object. This must be called before calling clutter_init().
- *
- * Since: 1.8
- * Stability: unstable
- */
-void
-clutter_wayland_set_compositor_display (void *display)
-{
-  if (_clutter_context_is_initialized ())
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  _wayland_compositor_display = display;
-}
-#endif
-
-/**
- * clutter_set_windowing_backend:
- * @backend_type: the name of a clutter window backend
- *
- * Restricts clutter to only use the specified backend.
- * This must be called before the first API call to clutter, including
- * clutter_get_option_context()
- *
- * Since: 1.16
- */
-void
-clutter_set_windowing_backend (const char *backend_type)
-{
-  g_return_if_fail (backend_type != NULL);
-
-  allowed_backend = g_intern_string (backend_type);
-}
-
-PangoDirection
-_clutter_backend_get_keymap_direction (ClutterBackend *backend)
-{
-  ClutterBackendClass *klass;
-
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (klass->get_keymap_direction != NULL)
-    return klass->get_keymap_direction (backend);
-
-  return PANGO_DIRECTION_NEUTRAL;
-}
-
-void
-_clutter_backend_reset_cogl_framebuffer (ClutterBackend *backend)
-{
-  if (backend->dummy_onscreen == COGL_INVALID_HANDLE)
-    {
-      CoglError *internal_error = NULL;
-
-      backend->dummy_onscreen = cogl_onscreen_new (backend->cogl_context, 1, 1);
-
-      if (!cogl_framebuffer_allocate (COGL_FRAMEBUFFER (backend->dummy_onscreen),
-                                      &internal_error))
-        {
-          g_critical ("Unable to create dummy onscreen: %s", internal_error->message);
-          cogl_error_free (internal_error);
-          return;
-        }
-    }
-
-  cogl_set_framebuffer (COGL_FRAMEBUFFER (backend->dummy_onscreen));
+  priv->event_translators =
+    g_list_remove (priv->event_translators, translator);
 }
