@@ -54,20 +54,6 @@
  * parented and exist between the emission of #ClutterDragAction::drag-begin
  * and #ClutterDragAction::drag-end.
  *
- * <example id="drag-action-example">
- *   <title>A simple draggable actor</title>
- *   <programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/drag-action.c">
- *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
- * </xi:include>
- *   </programlisting>
- *   <para>The example program above allows dragging the rectangle around
- *   the stage using a #ClutterDragAction. When pressing the
- *   <keycap>Shift</keycap> key the actor that is going to be dragged is a
- *   separate rectangle, and when the drag ends, the original rectangle will
- *   be animated to the final coordinates.</para>
- * </example>
- *
  * #ClutterDragAction is available since Clutter 1.4
  */
 
@@ -91,12 +77,8 @@ struct _ClutterDragActionPrivate
   gint y_drag_threshold;
   ClutterActor *drag_handle;
   ClutterDragAxis drag_axis;
-  ClutterRect drag_area;
 
-  ClutterInputDevice *device;
-  ClutterEventSequence *sequence;
   gulong button_press_id;
-  gulong touch_begin_id;
   gulong capture_id;
 
   gfloat press_x;
@@ -105,8 +87,6 @@ struct _ClutterDragActionPrivate
 
   gfloat last_motion_x;
   gfloat last_motion_y;
-  ClutterModifierType last_motion_state;
-  ClutterInputDevice *last_motion_device;
 
   gfloat transformed_press_x;
   gfloat transformed_press_y;
@@ -114,7 +94,6 @@ struct _ClutterDragActionPrivate
   guint emit_delayed_press    : 1;
   guint in_drag               : 1;
   guint motion_events_enabled : 1;
-  guint drag_area_set         : 1;
 };
 
 enum
@@ -125,8 +104,6 @@ enum
   PROP_Y_DRAG_THRESHOLD,
   PROP_DRAG_HANDLE,
   PROP_DRAG_AXIS,
-  PROP_DRAG_AREA,
-  PROP_DRAG_AREA_SET,
 
   PROP_LAST
 };
@@ -136,7 +113,6 @@ static GParamSpec *drag_props[PROP_LAST] = { NULL, };
 enum
 {
   DRAG_BEGIN,
-  DRAG_PROGRESS,
   DRAG_MOTION,
   DRAG_END,
 
@@ -150,7 +126,7 @@ static gboolean on_captured_event (ClutterActor      *stage,
                                    ClutterEvent      *event,
                                    ClutterDragAction *action);
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION)
+G_DEFINE_TYPE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION);
 
 static void
 get_drag_threshold (ClutterDragAction *action,
@@ -188,21 +164,7 @@ emit_drag_begin (ClutterDragAction *action,
   ClutterDragActionPrivate *priv = action->priv;
 
   if (priv->stage != NULL)
-    {
-      clutter_stage_set_motion_events_enabled (priv->stage, FALSE);
-      if (clutter_event_type (event) == CLUTTER_TOUCH_BEGIN)
-        _clutter_stage_add_touch_drag_actor (priv->stage,
-                                             clutter_event_get_event_sequence (event),
-                                             priv->drag_handle != NULL
-                                             ? priv->drag_handle
-                                             : actor);
-      else
-        _clutter_stage_add_pointer_drag_actor (priv->stage,
-                                               clutter_event_get_device (event),
-                                               priv->drag_handle != NULL
-                                               ? priv->drag_handle
-                                               : actor);
-    }
+    _clutter_stage_set_motion_events_enabled (priv->stage, FALSE);
 
   g_signal_emit (action, drag_signals[DRAG_BEGIN], 0,
                  actor,
@@ -219,11 +181,8 @@ emit_drag_motion (ClutterDragAction *action,
   ClutterActor *drag_handle = NULL;
   gfloat delta_x, delta_y;
   gfloat motion_x, motion_y;
-  gboolean can_emit_drag_motion = TRUE;
 
   clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
-  priv->last_motion_state = clutter_event_get_state (event);
-  priv->last_motion_device = clutter_event_get_device (event);
 
   if (priv->drag_handle != NULL && !priv->emit_delayed_press)
     drag_handle = priv->drag_handle;
@@ -269,23 +228,15 @@ emit_drag_motion (ClutterDragAction *action,
         {
           priv->emit_delayed_press = FALSE;
 
-          emit_drag_begin (action, actor, event);
+          emit_drag_begin (action, actor, NULL);
         }
       else
         return;
     }
 
-  g_signal_emit (action, drag_signals[DRAG_PROGRESS], 0,
+  g_signal_emit (action, drag_signals[DRAG_MOTION], 0,
                  actor,
-                 delta_x, delta_y,
-                 &can_emit_drag_motion);
-
-  if (can_emit_drag_motion)
-    {
-      g_signal_emit (action, drag_signals[DRAG_MOTION], 0,
-                     actor,
-                     delta_x, delta_y);
-    }
+                 delta_x, delta_y);
 }
 
 static void
@@ -295,34 +246,14 @@ emit_drag_end (ClutterDragAction *action,
 {
   ClutterDragActionPrivate *priv = action->priv;
 
-  /* ::drag-end may result in the destruction of the actor, which in turn
-   * will lead to the removal and finalization of the action, so we need
-   * to keep the action alive for the entire emission sequence
-   */
-  g_object_ref (action);
-
-  /* if we have an event, update our own state, otherwise we'll
-   * just use the currently stored state when emitting the ::drag-end
-   * signal
-   */
-  if (event != NULL)
-    {
-      clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
-      priv->last_motion_state = clutter_event_get_state (event);
-      priv->last_motion_device = clutter_event_get_device (event);
-    }
-
-  priv->in_drag = FALSE;
+  clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
 
   /* we might not have emitted ::drag-begin yet */
   if (!priv->emit_delayed_press)
     g_signal_emit (action, drag_signals[DRAG_END], 0,
                    actor,
                    priv->last_motion_x, priv->last_motion_y,
-                   priv->last_motion_state);
-
-  if (priv->stage == NULL)
-    goto out;
+                   clutter_event_get_state (event));
 
   /* disconnect the capture */
   if (priv->capture_id != 0)
@@ -331,24 +262,10 @@ emit_drag_end (ClutterDragAction *action,
       priv->capture_id = 0;
     }
 
-  clutter_stage_set_motion_events_enabled (priv->stage,
-                                           priv->motion_events_enabled);
+  _clutter_stage_set_motion_events_enabled (priv->stage,
+                                            priv->motion_events_enabled);
 
-  if (priv->last_motion_device != NULL && event != NULL)
-    {
-      if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE)
-        _clutter_stage_remove_pointer_drag_actor (priv->stage,
-                                                  priv->last_motion_device);
-      else
-        _clutter_stage_remove_touch_drag_actor (priv->stage,
-                                                priv->sequence);
-    }
-
-out:
-  priv->last_motion_device = NULL;
-  priv->sequence = NULL;
-
-  g_object_unref (action);
+  priv->in_drag = FALSE;
 }
 
 static gboolean
@@ -358,22 +275,14 @@ on_captured_event (ClutterActor      *stage,
 {
   ClutterDragActionPrivate *priv = action->priv;
   ClutterActor *actor;
-
+  
   actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
 
   if (!priv->in_drag)
-    return CLUTTER_EVENT_PROPAGATE;
-
-  if (clutter_event_get_device (event) != priv->device ||
-      clutter_event_get_event_sequence (event) != priv->sequence)
-    return CLUTTER_EVENT_PROPAGATE;
+    return FALSE;
 
   switch (clutter_event_type (event))
     {
-    case CLUTTER_TOUCH_UPDATE:
-      emit_drag_motion (action, actor, event);
-      break;
-
     case CLUTTER_MOTION:
       {
         ClutterModifierType mods = clutter_event_get_state (event);
@@ -389,11 +298,6 @@ on_captured_event (ClutterActor      *stage,
       }
       break;
 
-    case CLUTTER_TOUCH_END:
-    case CLUTTER_TOUCH_CANCEL:
-      emit_drag_end (action, actor, event);
-      break;
-
     case CLUTTER_BUTTON_RELEASE:
       if (priv->in_drag)
         emit_drag_end (action, actor, event);
@@ -402,45 +306,28 @@ on_captured_event (ClutterActor      *stage,
     case CLUTTER_ENTER:
     case CLUTTER_LEAVE:
       if (priv->in_drag)
-        return CLUTTER_EVENT_STOP;
+        return TRUE;
       break;
 
     default:
       break;
     }
 
-  return CLUTTER_EVENT_PROPAGATE;
+  return FALSE;
 }
 
 static gboolean
-on_drag_begin (ClutterActor      *actor,
-               ClutterEvent      *event,
-               ClutterDragAction *action)
+on_button_press (ClutterActor      *actor,
+                 ClutterEvent      *event,
+                 ClutterDragAction *action)
 {
   ClutterDragActionPrivate *priv = action->priv;
 
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (action)))
-    return CLUTTER_EVENT_PROPAGATE;
+    return FALSE;
 
-  /* dragging is only performed using the primary button */
-  switch (clutter_event_type (event))
-    {
-    case CLUTTER_BUTTON_PRESS:
-      if (priv->sequence != NULL)
-        return CLUTTER_EVENT_PROPAGATE;
-      if (clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
-        return CLUTTER_EVENT_PROPAGATE;
-      break;
-
-    case CLUTTER_TOUCH_BEGIN:
-      if (priv->sequence != NULL)
-        return CLUTTER_EVENT_PROPAGATE;
-      priv->sequence = clutter_event_get_event_sequence (event);
-      break;
-
-    default:
-      return CLUTTER_EVENT_PROPAGATE;
-    }
+  if (clutter_event_get_button (event) != 1)
+    return FALSE;
 
   if (priv->stage == NULL)
     priv->stage = CLUTTER_STAGE (clutter_actor_get_stage (actor));
@@ -448,8 +335,6 @@ on_drag_begin (ClutterActor      *actor,
   clutter_event_get_coords (event, &priv->press_x, &priv->press_y);
   priv->press_state = clutter_event_get_state (event);
 
-  priv->device = clutter_event_get_device (event);
-
   priv->last_motion_x = priv->press_x;
   priv->last_motion_y = priv->press_y;
 
@@ -460,7 +345,7 @@ on_drag_begin (ClutterActor      *actor,
                                        &priv->transformed_press_y);
 
   priv->motion_events_enabled =
-    clutter_stage_get_motion_events_enabled (priv->stage);
+    _clutter_stage_get_motion_events_enabled (priv->stage);
 
   if (priv->x_drag_threshold == 0 || priv->y_drag_threshold == 0)
     emit_drag_begin (action, actor, event);
@@ -472,7 +357,7 @@ on_drag_begin (ClutterActor      *actor,
                                              G_CALLBACK (on_captured_event),
                                              action);
 
-  return CLUTTER_EVENT_PROPAGATE;
+  return FALSE;
 }
 
 static void
@@ -486,51 +371,26 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
       ClutterActor *old_actor;
 
       old_actor = clutter_actor_meta_get_actor (meta);
-      if (old_actor != NULL)
-        {
-          g_signal_handler_disconnect (old_actor, priv->button_press_id);
-          g_signal_handler_disconnect (old_actor, priv->touch_begin_id);
-        }
+
+      g_signal_handler_disconnect (old_actor, priv->button_press_id);
+
+      if (priv->capture_id != 0)
+        g_signal_handler_disconnect (old_actor, priv->capture_id);
 
       priv->button_press_id = 0;
-      priv->touch_begin_id = 0;
-    }
-
-  if (priv->capture_id != 0)
-    {
-      if (priv->stage != NULL)
-        g_signal_handler_disconnect (priv->stage, priv->capture_id);
-
       priv->capture_id = 0;
+
       priv->stage = NULL;
     }
 
-  clutter_drag_action_set_drag_handle (CLUTTER_DRAG_ACTION (meta), NULL);
-
-  priv->in_drag = FALSE;
-
   if (actor != NULL)
-    {
-      priv->button_press_id = g_signal_connect (actor, "button-press-event",
-                                                G_CALLBACK (on_drag_begin),
-                                                meta);
-      priv->touch_begin_id = g_signal_connect (actor, "touch-event",
-                                               G_CALLBACK (on_drag_begin),
-                                               meta);
-    }
+    priv->button_press_id = g_signal_connect (actor, "button-press-event",
+                                              G_CALLBACK (on_button_press),
+                                              meta);
 
   CLUTTER_ACTOR_META_CLASS (clutter_drag_action_parent_class)->set_actor (meta, actor);
 }
 
-static gboolean
-clutter_drag_action_real_drag_progress (ClutterDragAction *action,
-                                        ClutterActor      *actor,
-                                        gfloat             delta_x,
-                                        gfloat             delta_y)
-{
-  return TRUE;
-}
-
 static void
 clutter_drag_action_real_drag_motion (ClutterDragAction *action,
                                       ClutterActor      *actor,
@@ -538,27 +398,13 @@ clutter_drag_action_real_drag_motion (ClutterDragAction *action,
                                       gfloat             delta_y)
 {
   ClutterActor *drag_handle;
-  gfloat x, y;
 
   if (action->priv->drag_handle != NULL)
     drag_handle = action->priv->drag_handle;
   else
     drag_handle = actor;
 
-  clutter_actor_get_position (drag_handle, &x, &y);
-
-  x += delta_x;
-  y += delta_y;
-
-  if (action->priv->drag_area_set)
-    {
-      ClutterRect *drag_area = &action->priv->drag_area;
-
-      x = CLAMP (x, drag_area->origin.x, drag_area->origin.x + drag_area->size.width);
-      y = CLAMP (y, drag_area->origin.y, drag_area->origin.y + drag_area->size.height);
-    }
-
-  clutter_actor_set_position (drag_handle, x, y);
+  clutter_actor_move_by (drag_handle, delta_x, delta_y);
 }
 
 static void
@@ -592,10 +438,6 @@ clutter_drag_action_set_property (GObject      *gobject,
       clutter_drag_action_set_drag_axis (action, g_value_get_enum (value));
       break;
 
-    case PROP_DRAG_AREA:
-      clutter_drag_action_set_drag_area (action, g_value_get_boxed (value));
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -637,14 +479,6 @@ clutter_drag_action_get_property (GObject    *gobject,
       g_value_set_enum (value, priv->drag_axis);
       break;
 
-    case PROP_DRAG_AREA:
-      g_value_set_boxed (value, &priv->drag_area);
-      break;
-
-    case PROP_DRAG_AREA_SET:
-      g_value_set_boolean (value, priv->drag_area_set);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -655,28 +489,8 @@ clutter_drag_action_dispose (GObject *gobject)
 {
   ClutterDragActionPrivate *priv = CLUTTER_DRAG_ACTION (gobject)->priv;
 
-  /* if we're being disposed while a capture is still present, we
-   * need to reset the state we are currently holding
-   */
-  if (priv->last_motion_device != NULL)
-    {
-      _clutter_stage_remove_pointer_drag_actor (priv->stage,
-                                                priv->last_motion_device);
-      priv->last_motion_device = NULL;
-    }
-
-  if (priv->sequence != NULL)
-    {
-      _clutter_stage_remove_touch_drag_actor (priv->stage,
-                                              priv->sequence);
-      priv->sequence = NULL;
-    }
-
   if (priv->capture_id != 0)
     {
-      clutter_stage_set_motion_events_enabled (priv->stage,
-                                               priv->motion_events_enabled);
-
       if (priv->stage != NULL)
         g_signal_handler_disconnect (priv->stage, priv->capture_id);
 
@@ -689,19 +503,10 @@ clutter_drag_action_dispose (GObject *gobject)
       ClutterActor *actor;
 
       actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (gobject));
-      if (actor != NULL)
-        {
-          g_signal_handler_disconnect (actor, priv->button_press_id);
-          g_signal_handler_disconnect (actor, priv->touch_begin_id);
-        }
-
+      g_signal_handler_disconnect (actor, priv->button_press_id);
       priv->button_press_id = 0;
-      priv->touch_begin_id = 0;
     }
 
-  clutter_drag_action_set_drag_handle (CLUTTER_DRAG_ACTION (gobject), NULL);
-  clutter_drag_action_set_drag_area (CLUTTER_DRAG_ACTION (gobject), NULL);
-
   G_OBJECT_CLASS (clutter_drag_action_parent_class)->dispose (gobject);
 }
 
@@ -711,9 +516,10 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (ClutterDragActionPrivate));
+
   meta_class->set_actor = clutter_drag_action_set_actor;
 
-  klass->drag_progress = clutter_drag_action_real_drag_progress;
   klass->drag_motion = clutter_drag_action_real_drag_motion;
 
   /**
@@ -808,37 +614,6 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                        CLUTTER_DRAG_AXIS_NONE,
                        CLUTTER_PARAM_READWRITE);
 
-  /**
-   * ClutterDragAction:drag-area:
-   *
-   * Constains the dragging action (or in particular, the resulting
-   * actor position) to the specified #ClutterRect, in parent's
-   * coordinates.
-   *
-   * Since: 1.12
-   */
-  drag_props[PROP_DRAG_AREA] =
-    g_param_spec_boxed ("drag-area",
-			P_("Drag Area"),
-			P_("Constrains the dragging to a rectangle"),
-			CLUTTER_TYPE_RECT,
-			CLUTTER_PARAM_READWRITE);
-
-  /**
-   * ClutterDragAction:drag-area-set:
-   *
-   * Whether the #ClutterDragAction:drag-area property has been set.
-   *
-   * Since: 1.12
-   */
-  drag_props[PROP_DRAG_AREA_SET] =
-    g_param_spec_boolean ("drag-area-set",
-			  P_("Drag Area Set"),
-			  P_("Whether the drag area is set"),
-			  FALSE,
-			  CLUTTER_PARAM_READABLE);
-
-
   gobject_class->set_property = clutter_drag_action_set_property;
   gobject_class->get_property = clutter_drag_action_get_property;
   gobject_class->dispose = clutter_drag_action_dispose;
@@ -877,47 +652,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                   CLUTTER_TYPE_MODIFIER_TYPE);
 
   /**
-   * ClutterDragAction::drag-progress:
-   * @action: the #ClutterDragAction that emitted the signal
-   * @actor: the #ClutterActor attached to the action
-   * @delta_x: the X component of the distance between the press event
-   *   that began the dragging and the current position of the pointer,
-   *   as of the latest motion event
-   * @delta_y: the Y component of the distance between the press event
-   *   that began the dragging and the current position of the pointer,
-   *   as of the latest motion event
-   *
-   * The ::drag-progress signal is emitted for each motion event after
-   * the #ClutterDragAction::drag-begin signal has been emitted.
-   *
-   * The components of the distance between the press event and the
-   * latest motion event are computed in the actor's coordinate space,
-   * to take into account eventual transformations. If you want the
-   * stage coordinates of the latest motion event you can use
-   * clutter_drag_action_get_motion_coords().
-   *
-   * The default handler will emit #ClutterDragAction::drag-motion,
-   * if #ClutterDragAction::drag-progress emission returns %TRUE.
-   *
-   * Return value: %TRUE if the drag should continue, and %FALSE
-   *   if it should be stopped.
-   *
-   * Since: 1.12
-   */
-  drag_signals[DRAG_PROGRESS] =
-    g_signal_new (I_("drag-progress"),
-                  CLUTTER_TYPE_DRAG_ACTION,
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (ClutterDragActionClass, drag_progress),
-                  _clutter_boolean_continue_accumulator, NULL,
-                  _clutter_marshal_BOOLEAN__OBJECT_FLOAT_FLOAT,
-                  G_TYPE_BOOLEAN, 3,
-                  CLUTTER_TYPE_ACTOR,
-                  G_TYPE_FLOAT,
-                  G_TYPE_FLOAT);
-
-  /**
-   * ClutterDragAction::drag-motion:
+   * ClutterDragAction::drag-motion
    * @action: the #ClutterDragAction that emitted the signal
    * @actor: the #ClutterActor attached to the action
    * @delta_x: the X component of the distance between the press event
@@ -939,9 +674,9 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * The default handler of the signal will call clutter_actor_move_by()
    * either on @actor or, if set, of #ClutterDragAction:drag-handle using
    * the @delta_x and @delta_y components of the dragging motion. If you
-   * want to override the default behaviour, you can connect to the
-   * #ClutterDragAction::drag-progress signal and return %FALSE from the
-   * handler.
+   * want to override the default behaviour, you can connect to this
+   * signal and call g_signal_stop_emission_by_name() from within your
+   * callback.
    *
    * Since: 1.4
    */
@@ -990,7 +725,8 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 static void
 clutter_drag_action_init (ClutterDragAction *self)
 {
-  self->priv = clutter_drag_action_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_DRAG_ACTION,
+                                            ClutterDragActionPrivate);
 }
 
 /**
@@ -1092,27 +828,12 @@ clutter_drag_action_get_drag_threshold (ClutterDragAction *action,
     *y_threshold = y_res;
 }
 
-static void
-on_drag_handle_destroy (ClutterActor      *handle,
-                        ClutterDragAction *action)
-{
-  ClutterDragActionPrivate *priv = action->priv;
-  ClutterActor *actor;
-
-  /* make sure we reset the state */
-  actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
-  if (priv->in_drag)
-    emit_drag_end (action, actor, NULL);
-
-  priv->drag_handle = NULL;
-}
-
 /**
  * clutter_drag_action_set_drag_handle:
  * @action: a #ClutterDragAction
- * @handle: (allow-none): a #ClutterActor, or %NULL to unset
+ * @handle: a #ClutterActor
  *
- * Sets the actor to be used as the drag handle.
+ * Sets the actor to be used as the drag handle
  *
  * Since: 1.4
  */
@@ -1123,35 +844,15 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
   ClutterDragActionPrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
-  g_return_if_fail (handle == NULL || CLUTTER_IS_ACTOR (handle));
+  g_return_if_fail (CLUTTER_IS_ACTOR (handle));
 
   priv = action->priv;
 
   if (priv->drag_handle == handle)
     return;
 
-  if (priv->drag_handle != NULL)
-    g_signal_handlers_disconnect_by_func (priv->drag_handle,
-                                          G_CALLBACK (on_drag_handle_destroy),
-                                          action);
-
   priv->drag_handle = handle;
 
-  priv->transformed_press_x = priv->press_x;
-  priv->transformed_press_y = priv->press_y;
-
-  if (priv->drag_handle != NULL)
-    {
-      clutter_actor_transform_stage_point (priv->drag_handle,
-                                           priv->press_x,
-                                           priv->press_y,
-					   &priv->transformed_press_x,
-					   &priv->transformed_press_y);
-      g_signal_connect (priv->drag_handle, "destroy",
-			G_CALLBACK (on_drag_handle_destroy),
-			action);
-    }
-
   g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_HANDLE]);
 }
 
@@ -1273,58 +974,3 @@ clutter_drag_action_get_motion_coords (ClutterDragAction *action,
   if (motion_y)
     *motion_y = action->priv->last_motion_y;
 }
-
-/**
- * clutter_drag_action_get_drag_area:
- * @action: a #ClutterDragAction
- * @drag_area: (out caller-allocates): a #ClutterRect to be filled
- *
- * Retrieves the "drag area" associated with @action, that
- * is a #ClutterRect that constrains the actor movements,
- * in parents coordinates.
- *
- * Returns: %TRUE if the actor is actually constrained (and thus
- *          @drag_area is valid), %FALSE otherwise
- */
-gboolean
-clutter_drag_action_get_drag_area (ClutterDragAction *action,
-				   ClutterRect       *drag_area)
-{
-  g_return_val_if_fail (CLUTTER_IS_DRAG_ACTION (action), FALSE);
-
-  if (drag_area != NULL)
-    *drag_area = action->priv->drag_area;
-  return action->priv->drag_area_set;
-}
-
-/**
- * clutter_drag_action_set_drag_area:
- * @action: a #ClutterDragAction
- * @drag_area: (allow-none): a #ClutterRect
- *
- * Sets @drag_area to constrain the dragging of the actor associated
- * with @action, so that it position is always within @drag_area, expressed
- * in parent's coordinates.
- * If @drag_area is %NULL, the actor is not constrained.
- */
-void
-clutter_drag_action_set_drag_area (ClutterDragAction *action,
-				   const ClutterRect *drag_area)
-{
-  ClutterDragActionPrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
-
-  priv = action->priv;
-
-  if (drag_area != NULL)
-    {
-      priv->drag_area = *drag_area;
-      priv->drag_area_set = TRUE;
-    }
-  else
-    priv->drag_area_set = FALSE;
-
-  g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_AREA_SET]);
-  g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_AREA]);
-}
