@@ -48,21 +48,21 @@
  *   in both columns and rows.</para></listitem>
  * </itemizedlist>
  *
- * <figure id="flow-layout-image">
+ * <figure id="flow-layout-horizontal">
  *   <title>Horizontal flow layout</title>
  *   <para>The image shows a #ClutterFlowLayout with the
  *   #ClutterFlowLayout:orientation propert set to
  *   %CLUTTER_FLOW_HORIZONTAL.</para>
- *   <graphic fileref="flow-layout.png" format="PNG"/>
+ *   <graphic fileref="flow-layout-horizontal.png" format="PNG"/>
  * </figure>
  *
- * <informalexample>
- *  <programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/flow-layout.c">
- *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
- * </xi:include>
- *  </programlisting>
- * </informalexample>
+ * <figure id="flow-layout-vertical">
+ *   <title>Vertical flow layout</title>
+ *   <para>The image shows a #ClutterFlowLayout with the
+ *   #ClutterFlowLayout:orientation propert set to
+ *   %CLUTTER_FLOW_VERTICAL.</para>
+ *   <graphic fileref="flow-layout-vertical.png" format="PNG"/>
+ * </figure>
  *
  * #ClutterFlowLayout is available since Clutter 1.2
  */
@@ -73,9 +73,6 @@
 
 #include <math.h>
 
-#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
-#include "deprecated/clutter-container.h"
-
 #include "clutter-actor.h"
 #include "clutter-animatable.h"
 #include "clutter-child-meta.h"
@@ -85,6 +82,8 @@
 #include "clutter-layout-meta.h"
 #include "clutter-private.h"
 
+#define CLUTTER_FLOW_LAYOUT_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_FLOW_LAYOUT, ClutterFlowLayoutPrivate))
+
 struct _ClutterFlowLayoutPrivate
 {
   ClutterContainer *container;
@@ -111,7 +110,6 @@ struct _ClutterFlowLayoutPrivate
   guint line_count;
 
   guint is_homogeneous : 1;
-  guint snap_to_grid : 1;
 };
 
 enum
@@ -130,16 +128,14 @@ enum
   PROP_MIN_ROW_HEGHT,
   PROP_MAX_ROW_HEIGHT,
 
-  PROP_SNAP_TO_GRID,
-
   N_PROPERTIES
 };
 
 static GParamSpec *flow_properties[N_PROPERTIES] = { NULL, };
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterFlowLayout,
-                            clutter_flow_layout,
-                            CLUTTER_TYPE_LAYOUT_MANAGER)
+G_DEFINE_TYPE (ClutterFlowLayout,
+               clutter_flow_layout,
+               CLUTTER_TYPE_LAYOUT_MANAGER);
 
 static gint
 get_columns (ClutterFlowLayout *self,
@@ -206,12 +202,11 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
                                          gfloat               *nat_width_p)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
+  GList *l, *children = clutter_container_get_children (container);
   gint n_rows, line_item_count, line_count;
   gfloat total_min_width, total_natural_width;
   gfloat line_min_width, line_natural_width;
   gfloat max_min_width, max_natural_width;
-  ClutterActor *actor, *child;
-  ClutterActorIter iter;
   gfloat item_y;
 
   n_rows = get_rows (CLUTTER_FLOW_LAYOUT (manager), for_height);
@@ -227,8 +222,6 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
 
   item_y = 0;
 
-  actor = CLUTTER_ACTOR (container);
-
   /* clear the line width arrays */
   if (priv->line_min != NULL)
     g_array_free (priv->line_min, TRUE);
@@ -243,14 +236,14 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
                                           sizeof (gfloat),
                                           16);
 
-  if (clutter_actor_get_n_children (actor) != 0)
+  if (children)
     line_count = 1;
 
   max_min_width = max_natural_width = 0;
 
-  clutter_actor_iter_init (&iter, actor);
-  while (clutter_actor_iter_next (&iter, &child))
+  for (l = children; l != NULL; l = l->next)
     {
+      ClutterActor *child = l->data;
       gfloat child_min, child_natural;
       gfloat new_y, item_height;
 
@@ -259,12 +252,7 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
 
       if (priv->orientation == CLUTTER_FLOW_VERTICAL && for_height > 0)
         {
-          clutter_actor_get_preferred_height (child, -1,
-                                              &child_min,
-                                              &child_natural);
-
-          if ((priv->snap_to_grid && line_item_count == n_rows) ||
-              (!priv->snap_to_grid && item_y + child_natural > for_height))
+          if (line_item_count == n_rows)
             {
               total_min_width += line_min_width;
               total_natural_width += line_natural_width;
@@ -281,17 +269,9 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
               item_y = 0;
             }
 
-          if (priv->snap_to_grid)
-            {
-              new_y = ((line_item_count + 1) * (for_height + priv->row_spacing))
-                    / n_rows;
-              item_height = new_y - item_y - priv->row_spacing;
-            }
-          else
-            {
-              new_y = item_y + child_natural + priv->row_spacing;
-              item_height = child_natural;
-            }
+          new_y = ((line_item_count + 1) * (for_height + priv->row_spacing))
+                / n_rows;
+          item_height = new_y - item_y - priv->row_spacing;
 
           clutter_actor_get_preferred_width (child, item_height,
                                              &child_min,
@@ -321,6 +301,8 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
         }
     }
 
+  g_list_free (children);
+
   priv->col_width = max_natural_width;
 
   if (priv->max_col_width > 0 && priv->col_width > priv->max_col_width)
@@ -383,7 +365,7 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
   priv->req_height = for_height;
 
   if (min_width_p)
-    *min_width_p = max_min_width;
+    *min_width_p = total_min_width;
 
   if (nat_width_p)
     *nat_width_p = total_natural_width;
@@ -397,12 +379,11 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
                                           gfloat               *nat_height_p)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
+  GList *l, *children = clutter_container_get_children (container);
   gint n_columns, line_item_count, line_count;
   gfloat total_min_height, total_natural_height;
   gfloat line_min_height, line_natural_height;
   gfloat max_min_height, max_natural_height;
-  ClutterActor *actor, *child;
-  ClutterActorIter iter;
   gfloat item_x;
 
   n_columns = get_columns (CLUTTER_FLOW_LAYOUT (manager), for_width);
@@ -418,8 +399,6 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
 
   item_x = 0;
 
-  actor = CLUTTER_ACTOR (container);
-
   /* clear the line height arrays */
   if (priv->line_min != NULL)
     g_array_free (priv->line_min, TRUE);
@@ -434,14 +413,14 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
                                           sizeof (gfloat),
                                           16);
 
-  if (clutter_actor_get_n_children (actor) != 0)
+  if (children)
     line_count = 1;
 
   max_min_height = max_natural_height = 0;
 
-  clutter_actor_iter_init (&iter, actor);
-  while (clutter_actor_iter_next (&iter, &child))
+  for (l = children; l != NULL; l = l->next)
     {
+      ClutterActor *child = l->data;
       gfloat child_min, child_natural;
       gfloat new_x, item_width;
 
@@ -450,12 +429,7 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
 
       if (priv->orientation == CLUTTER_FLOW_HORIZONTAL && for_width > 0)
         {
-          clutter_actor_get_preferred_width (child, -1,
-                                             &child_min,
-                                             &child_natural);
-
-          if ((priv->snap_to_grid && line_item_count == n_columns) ||
-              (!priv->snap_to_grid && item_x + child_natural > for_width))
+          if (line_item_count == n_columns)
             {
               total_min_height += line_min_height;
               total_natural_height += line_natural_height;
@@ -472,17 +446,9 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
               item_x = 0;
             }
 
-          if (priv->snap_to_grid)
-            {
-              new_x = ((line_item_count + 1) * (for_width + priv->col_spacing))
-                    / n_columns;
-              item_width = new_x - item_x - priv->col_spacing;
-            }
-          else
-            {
-              new_x = item_x + child_natural + priv->col_spacing;
-              item_width = child_natural;
-            }
+          new_x = ((line_item_count + 1) * (for_width + priv->col_spacing))
+                / n_columns;
+          item_width = new_x - item_x - priv->col_spacing;
 
           clutter_actor_get_preferred_height (child, item_width,
                                               &child_min,
@@ -513,6 +479,8 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
         }
     }
 
+  g_list_free (children);
+
   priv->row_height = max_natural_height;
 
   if (priv->max_row_height > 0 && priv->row_height > priv->max_row_height)
@@ -574,7 +542,7 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
   priv->req_width = for_width;
 
   if (min_height_p)
-    *min_height_p = max_min_height;
+    *min_height_p = total_min_height;
 
   if (nat_height_p)
     *nat_height_p = total_natural_height;
@@ -587,28 +555,24 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
                               ClutterAllocationFlags  flags)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
-  ClutterActor *actor, *child;
-  ClutterActorIter iter;
-  gfloat x_off, y_off;
+  GList *l, *children = clutter_container_get_children (container);
   gfloat avail_width, avail_height;
   gfloat item_x, item_y;
   gint line_item_count;
   gint items_per_line;
   gint line_index;
 
-  actor = CLUTTER_ACTOR (container);
-  if (clutter_actor_get_n_children (actor) == 0)
+  if (children == NULL)
     return;
 
-  clutter_actor_box_get_origin (allocation, &x_off, &y_off);
   clutter_actor_box_get_size (allocation, &avail_width, &avail_height);
 
   /* blow the cached preferred size and re-compute with the given
    * available size in case the FlowLayout wasn't given the exact
    * size it requested
    */
-  if ((priv->req_width >= 0 && avail_width != priv->req_width) ||
-      (priv->req_height >= 0 && avail_height != priv->req_height))
+  if ((priv->req_width > 0 && avail_width != priv->req_width) ||
+      (priv->req_height > 0 && avail_height != priv->req_height))
     {
       clutter_flow_layout_get_preferred_width (manager, container,
                                                avail_height,
@@ -621,36 +585,26 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
   items_per_line = compute_lines (CLUTTER_FLOW_LAYOUT (manager),
                                   avail_width, avail_height);
 
-  item_x = x_off;
-  item_y = y_off;
+  item_x = item_y = 0;
 
   line_item_count = 0;
   line_index = 0;
 
-  clutter_actor_iter_init (&iter, actor);
-  while (clutter_actor_iter_next (&iter, &child))
+  for (l = children; l != NULL; l = l->next)
     {
+      ClutterActor *child = l->data;
       ClutterActorBox child_alloc;
       gfloat item_width, item_height;
       gfloat new_x, new_y;
-      gfloat child_min, child_natural;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
       new_x = new_y = 0;
 
-      if (!priv->snap_to_grid)
-        clutter_actor_get_preferred_size (child,
-                                          NULL, NULL,
-                                          &item_width,
-                                          &item_height);
-
       if (priv->orientation == CLUTTER_FLOW_HORIZONTAL)
         {
-          if ((priv->snap_to_grid &&
-               line_item_count == items_per_line && line_item_count > 0) ||
-              (!priv->snap_to_grid && item_x + item_width > avail_width))
+          if (line_item_count == items_per_line && line_item_count > 0)
             {
               item_y += g_array_index (priv->line_natural,
                                        gfloat,
@@ -662,30 +616,34 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_x = x_off;
-            }
-
-          if (priv->snap_to_grid)
-            {
-              new_x = x_off + ((line_item_count + 1) * (avail_width + priv->col_spacing))
-                    / items_per_line;
-              item_width = new_x - item_x - priv->col_spacing;
-            }
-          else
-            {
-              new_x = item_x + item_width + priv->col_spacing;
+              item_x = 0;
             }
 
+          new_x = ((line_item_count + 1) * (avail_width + priv->col_spacing))
+                / items_per_line;
+          item_width = new_x - item_x - priv->col_spacing;
           item_height = g_array_index (priv->line_natural,
                                        gfloat,
                                        line_index);
 
+          if (!priv->is_homogeneous)
+            {
+              gfloat child_min, child_natural;
+
+              clutter_actor_get_preferred_width (child, item_height,
+                                                 &child_min,
+                                                 &child_natural);
+              item_width = MIN (item_width, child_natural);
+
+              clutter_actor_get_preferred_height (child, item_width,
+                                                  &child_min,
+                                                  &child_natural);
+              item_height = MIN (item_height, child_natural);
+            }
         }
       else
         {
-          if ((priv->snap_to_grid &&
-               line_item_count == items_per_line && line_item_count > 0) ||
-              (!priv->snap_to_grid && item_y + item_height > avail_height))
+          if (line_item_count == items_per_line && line_item_count > 0)
             {
               item_x += g_array_index (priv->line_natural,
                                        gfloat,
@@ -697,43 +655,30 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_y = y_off;
-            }
-
-          if (priv->snap_to_grid)
-            {
-              new_y = y_off + ((line_item_count + 1) * (avail_height + priv->row_spacing))
-                    / items_per_line;
-              item_height = new_y - item_y - priv->row_spacing;
-            }
-          else
-            {
-              new_y = item_y + item_height + priv->row_spacing;
+              item_y = 0;
             }
 
+          new_y = ((line_item_count + 1) * (avail_height + priv->row_spacing))
+                / items_per_line;
+          item_height = new_y - item_y - priv->row_spacing;
           item_width = g_array_index (priv->line_natural,
                                       gfloat,
                                       line_index);
-        }
 
-      if (!priv->is_homogeneous &&
-          !clutter_actor_needs_expand (child,
-                                       CLUTTER_ORIENTATION_HORIZONTAL))
-        {
-          clutter_actor_get_preferred_width (child, item_height,
-                                             &child_min,
-                                             &child_natural);
-          item_width = MIN (item_width, child_natural);
-        }
+          if (!priv->is_homogeneous)
+            {
+              gfloat child_min, child_natural;
 
-      if (!priv->is_homogeneous &&
-          !clutter_actor_needs_expand (child,
-                                       CLUTTER_ORIENTATION_VERTICAL))
-        {
-          clutter_actor_get_preferred_height (child, item_width,
-                                              &child_min,
-                                              &child_natural);
-          item_height = MIN (item_height, child_natural);
+              clutter_actor_get_preferred_width (child, item_height,
+                                                 &child_min,
+                                                 &child_natural);
+              item_width = MIN (item_width, child_natural);
+
+              clutter_actor_get_preferred_height (child, item_width,
+                                                  &child_min,
+                                                  &child_natural);
+              item_height = MIN (item_height, child_natural);
+            }
         }
 
       CLUTTER_NOTE (LAYOUT,
@@ -755,6 +700,8 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
 
       line_item_count += 1;
     }
+
+  g_list_free (children);
 }
 
 static void
@@ -834,11 +781,6 @@ clutter_flow_layout_set_property (GObject      *gobject,
                                           g_value_get_float (value));
       break;
 
-    case PROP_SNAP_TO_GRID:
-      clutter_flow_layout_set_snap_to_grid (self,
-                                            g_value_get_boolean (value));
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -887,10 +829,6 @@ clutter_flow_layout_get_property (GObject    *gobject,
       g_value_set_float (value, priv->max_row_height);
       break;
 
-    case PROP_SNAP_TO_GRID:
-      g_value_set_boolean (value, priv->snap_to_grid);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -917,6 +855,8 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
   GObjectClass *gobject_class;
   ClutterLayoutManagerClass *layout_class;
 
+  g_type_class_add_private (klass, sizeof (ClutterFlowLayoutPrivate));
+
   gobject_class = G_OBJECT_CLASS (klass);
   layout_class = CLUTTER_LAYOUT_MANAGER_CLASS (klass);
 
@@ -1056,21 +996,6 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
                         -1.0,
                         CLUTTER_PARAM_READWRITE);
 
-  /**
-   * ClutterFlowLayout:snap-to-grid:
-   *
-   * Whether the #ClutterFlowLayout should arrange its children
-   * on a grid
-   *
-   * Since: 1.16
-   */
-  flow_properties[PROP_SNAP_TO_GRID] =
-    g_param_spec_boolean ("snap-to-grid",
-                          P_("Snap to grid"),
-                          P_("Snap to grid"),
-                          TRUE,
-                          CLUTTER_PARAM_READWRITE);
-
   gobject_class->finalize = clutter_flow_layout_finalize;
   gobject_class->set_property = clutter_flow_layout_set_property;
   gobject_class->get_property = clutter_flow_layout_get_property;
@@ -1084,7 +1009,7 @@ clutter_flow_layout_init (ClutterFlowLayout *self)
 {
   ClutterFlowLayoutPrivate *priv;
 
-  self->priv = priv = clutter_flow_layout_get_instance_private (self);
+  self->priv = priv = CLUTTER_FLOW_LAYOUT_GET_PRIVATE (self);
 
   priv->orientation = CLUTTER_FLOW_HORIZONTAL;
 
@@ -1096,7 +1021,6 @@ clutter_flow_layout_init (ClutterFlowLayout *self)
 
   priv->line_min = NULL;
   priv->line_natural = NULL;
-  priv->snap_to_grid = TRUE;
 }
 
 /**
@@ -1502,51 +1426,3 @@ clutter_flow_layout_get_row_height (ClutterFlowLayout *layout,
   if (max_height)
     *max_height = layout->priv->max_row_height;
 }
-
-/**
- * clutter_flow_layout_set_snap_to_grid:
- * @layout: a #ClutterFlowLayout
- * @snap_to_grid: %TRUE if @layout should place its children on a grid
- *
- * Whether the @layout should place its children on a grid.
- *
- * Since: 1.16
- */
-void
-clutter_flow_layout_set_snap_to_grid (ClutterFlowLayout *layout,
-                                      gboolean           snap_to_grid)
-{
-  ClutterFlowLayoutPrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_FLOW_LAYOUT (layout));
-
-  priv = layout->priv;
-
-  if (priv->snap_to_grid != snap_to_grid)
-    {
-      priv->snap_to_grid = snap_to_grid;
-
-      clutter_layout_manager_layout_changed (CLUTTER_LAYOUT_MANAGER (layout));
-
-      g_object_notify_by_pspec (G_OBJECT (layout),
-                                flow_properties[PROP_SNAP_TO_GRID]);
-    }
-}
-
-/**
- * clutter_flow_layout_get_snap_to_grid:
- * @layout: a #ClutterFlowLayout
- *
- * Retrieves the value of #ClutterFlowLayout:snap-to-grid property
- *
- * Return value: %TRUE if the @layout is placing its children on a grid
- *
- * Since: 1.16
- */
-gboolean
-clutter_flow_layout_get_snap_to_grid (ClutterFlowLayout *layout)
-{
-  g_return_val_if_fail (CLUTTER_IS_FLOW_LAYOUT (layout), FALSE);
-
-  return layout->priv->snap_to_grid;
-}
