@@ -31,7 +31,6 @@
 #include <string.h>
 
 #include <glib-object.h>
-#include <math.h>
 
 #include "clutter-actor-private.h"
 #include "clutter-paint-volume-private.h"
@@ -144,9 +143,7 @@ void
 _clutter_paint_volume_set_from_volume (ClutterPaintVolume       *pv,
                                        const ClutterPaintVolume *src)
 {
-  gboolean is_static = pv->is_static;
   memcpy (pv, src, sizeof (ClutterPaintVolume));
-  pv->is_static = is_static;
 }
 
 /**
@@ -191,6 +188,7 @@ clutter_paint_volume_set_origin (ClutterPaintVolume  *pv,
   int i;
 
   g_return_if_fail (pv != NULL);
+  g_return_if_fail (pv->is_axis_aligned);
 
   dx = origin->x - pv->vertices[0].x;
   dy = origin->y - pv->vertices[0].y;
@@ -243,8 +241,7 @@ _clutter_paint_volume_update_is_empty (ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @width: the width of the paint volume, in pixels
  *
- * Sets the width of the paint volume. The width is measured along
- * the x axis in the actor coordinates that @pv is associated with.
+ * Sets the width of the paint volume.
  *
  * Since: 1.6
  */
@@ -255,6 +252,7 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
   gfloat right_xpos;
 
   g_return_if_fail (pv != NULL);
+  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (width >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -262,9 +260,6 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
-  if (!pv->is_axis_aligned)
-    _clutter_paint_volume_axis_align (pv);
-
   right_xpos = pv->vertices[0].x + width;
 
   /* Move the right vertices of the paint box relative to the
@@ -283,30 +278,9 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_width:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the width of the volume's, axis aligned, bounding box.
+ * Retrieves the width set using clutter_paint_volume_get_width()
  *
- * In other words; this takes into account what actor's coordinate
- * space @pv belongs too and conceptually fits an axis aligned box
- * around the volume. It returns the size of that bounding box as
- * measured along the x-axis.
- *
- * If, for example, clutter_actor_get_transformed_paint_volume()
- * is used to transform a 2D child actor that is 100px wide, 100px
- * high and 0px deep into container coordinates then the width might
- * not simply be 100px if the child actor has a 3D rotation applied to
- * it.
- * 
- * Remember: if clutter_actor_get_transformed_paint_volume() is
- * used then a transformed child volume will be defined relative to the
- * ancestor container actor and so a 2D child actor can have a 3D
- * bounding volume.
- *
- * There are no accuracy guarantees for the reported width,
- * except that it must always be greater than, or equal to, the
- * actor's width. This is because actors may report simple, loose
- * fitting paint volumes for efficiency.
-
- * Return value: the width, in units of @pv's local coordinate system.
+ * Return value: the width, in pixels
  *
  * Since: 1.6
  */
@@ -314,19 +288,10 @@ gfloat
 clutter_paint_volume_get_width (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
+  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
-  else if (!pv->is_axis_aligned)
-    {
-      ClutterPaintVolume tmp;
-      float width;
-      _clutter_paint_volume_copy_static (pv, &tmp);
-      _clutter_paint_volume_axis_align (&tmp);
-      width = tmp.vertices[1].x - tmp.vertices[0].x;
-      clutter_paint_volume_free (&tmp);
-      return width;
-    }
   else
     return pv->vertices[1].x - pv->vertices[0].x;
 }
@@ -336,8 +301,7 @@ clutter_paint_volume_get_width (const ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @height: the height of the paint volume, in pixels
  *
- * Sets the height of the paint volume. The height is measured along
- * the y axis in the actor coordinates that @pv is associated with.
+ * Sets the height of the paint volume.
  *
  * Since: 1.6
  */
@@ -348,6 +312,7 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
   gfloat height_ypos;
 
   g_return_if_fail (pv != NULL);
+  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (height >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -355,9 +320,6 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
-  if (!pv->is_axis_aligned)
-    _clutter_paint_volume_axis_align (pv);
-
   height_ypos = pv->vertices[0].y + height;
 
   /* Move the bottom vertices of the paint box relative to the
@@ -375,30 +337,10 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_height:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the height of the volume's, axis aligned, bounding box.
+ * Retrieves the height of the paint volume set using
+ * clutter_paint_volume_get_height()
  *
- * In other words; this takes into account what actor's coordinate
- * space @pv belongs too and conceptually fits an axis aligned box
- * around the volume. It returns the size of that bounding box as
- * measured along the y-axis.
- *
- * If, for example, clutter_actor_get_transformed_paint_volume()
- * is used to transform a 2D child actor that is 100px wide, 100px
- * high and 0px deep into container coordinates then the height might
- * not simply be 100px if the child actor has a 3D rotation applied to
- * it.
- *
- * Remember: if clutter_actor_get_transformed_paint_volume() is
- * used then a transformed child volume will be defined relative to the
- * ancestor container actor and so a 2D child actor
- * can have a 3D bounding volume.
- *
- * There are no accuracy guarantees for the reported height,
- * except that it must always be greater than, or equal to, the actor's
- * height. This is because actors may report simple, loose fitting paint
- * volumes for efficiency.
- *
- * Return value: the height, in units of @pv's local coordinate system.
+ * Return value: the height of the paint volume, in pixels
  *
  * Since: 1.6
  */
@@ -406,19 +348,10 @@ gfloat
 clutter_paint_volume_get_height (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
+  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
-  else if (!pv->is_axis_aligned)
-    {
-      ClutterPaintVolume tmp;
-      float height;
-      _clutter_paint_volume_copy_static (pv, &tmp);
-      _clutter_paint_volume_axis_align (&tmp);
-      height = tmp.vertices[3].y - tmp.vertices[0].y;
-      clutter_paint_volume_free (&tmp);
-      return height;
-    }
   else
     return pv->vertices[3].y - pv->vertices[0].y;
 }
@@ -428,8 +361,7 @@ clutter_paint_volume_get_height (const ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @depth: the depth of the paint volume, in pixels
  *
- * Sets the depth of the paint volume. The depth is measured along
- * the z axis in the actor coordinates that @pv is associated with.
+ * Sets the depth of the paint volume.
  *
  * Since: 1.6
  */
@@ -440,6 +372,7 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
   gfloat depth_zpos;
 
   g_return_if_fail (pv != NULL);
+  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (depth >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -447,9 +380,6 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
-  if (!pv->is_axis_aligned)
-    _clutter_paint_volume_axis_align (pv);
-
   depth_zpos = pv->vertices[0].z + depth;
 
   /* Move the back vertices of the paint box relative to the
@@ -468,30 +398,10 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_depth:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the depth of the volume's, axis aligned, bounding box.
+ * Retrieves the depth of the paint volume set using
+ * clutter_paint_volume_get_depth()
  *
- * In other words; this takes into account what actor's coordinate
- * space @pv belongs too and conceptually fits an axis aligned box
- * around the volume. It returns the size of that bounding box as
- * measured along the z-axis.
- *
- * If, for example, clutter_actor_get_transformed_paint_volume()
- * is used to transform a 2D child actor that is 100px wide, 100px
- * high and 0px deep into container coordinates then the depth might
- * not simply be 0px if the child actor has a 3D rotation applied to
- * it.
- *
- * Remember: if clutter_actor_get_transformed_paint_volume() is
- * used then the transformed volume will be defined relative to the
- * container actor and in container coordinates a 2D child actor
- * can have a 3D bounding volume.
- *
- * There are no accuracy guarantees for the reported depth,
- * except that it must always be greater than, or equal to, the actor's
- * depth. This is because actors may report simple, loose fitting paint
- * volumes for efficiency.
- *
- * Return value: the depth, in units of @pv's local coordinate system.
+ * Return value: the depth
  *
  * Since: 1.6
  */
@@ -499,19 +409,10 @@ gfloat
 clutter_paint_volume_get_depth (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
+  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
-  else if (!pv->is_axis_aligned)
-    {
-      ClutterPaintVolume tmp;
-      float depth;
-      _clutter_paint_volume_copy_static (pv, &tmp);
-      _clutter_paint_volume_axis_align (&tmp);
-      depth = tmp.vertices[4].z - tmp.vertices[0].z;
-      clutter_paint_volume_free (&tmp);
-      return depth;
-    }
   else
     return pv->vertices[4].z - pv->vertices[0].z;
 }
@@ -522,10 +423,8 @@ clutter_paint_volume_get_depth (const ClutterPaintVolume *pv)
  *      union
  * @another_pv: A second #ClutterPaintVolume to union with @pv
  *
- * Updates the geometry of @pv to encompass @pv and @another_pv.
- *
- * There are no guarantees about how precisely the two volumes
- * will be unioned.
+ * Updates the geometry of @pv to be the union bounding box that
+ * encompases @pv and @another_pv.
  *
  * Since: 1.6
  */
@@ -534,13 +433,12 @@ clutter_paint_volume_union (ClutterPaintVolume *pv,
                             const ClutterPaintVolume *another_pv)
 {
   ClutterPaintVolume aligned_pv;
+  static const int key_vertices[4] = { 0, 1, 3, 4 };
 
   g_return_if_fail (pv != NULL);
+  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (another_pv != NULL);
 
-  /* Both volumes have to belong to the same local coordinate space */
-  g_return_if_fail (pv->actor == another_pv->actor);
-
   /* NB: we only have to update vertices 0, 1, 3 and 4
    * (See the ClutterPaintVolume typedef for more details) */
 
@@ -553,13 +451,13 @@ clutter_paint_volume_union (ClutterPaintVolume *pv,
 
   if (pv->is_empty)
     {
-      _clutter_paint_volume_set_from_volume (pv, another_pv);
+      int i;
+      for (i = 0; i < 4; i++)
+        pv->vertices[key_vertices[i]] = another_pv->vertices[key_vertices[i]];
+      pv->is_2d = another_pv->is_2d;
       goto done;
     }
 
-  if (!pv->is_axis_aligned)
-    _clutter_paint_volume_axis_align (pv);
-
   if (!another_pv->is_axis_aligned)
     {
       _clutter_paint_volume_copy_static (another_pv, &aligned_pv);
@@ -643,42 +541,6 @@ done:
   pv->is_complete = FALSE;
 }
 
-/**
- * clutter_paint_volume_union_box:
- * @pv: a #ClutterPaintVolume
- * @box: a #ClutterActorBox to union to @pv
- *
- * Unions the 2D region represented by @box to a #ClutterPaintVolume.
- *
- * This function is similar to clutter_paint_volume_union(), but it is
- * specific for 2D regions.
- *
- * Since: 1.10
- */
-void
-clutter_paint_volume_union_box (ClutterPaintVolume    *pv,
-                                const ClutterActorBox *box)
-{
-  ClutterPaintVolume volume;
-  ClutterVertex origin;
-
-  g_return_if_fail (pv != NULL);
-  g_return_if_fail (box != NULL);
-
-  _clutter_paint_volume_init_static (&volume, pv->actor);
-
-  origin.x = box->x1;
-  origin.y = box->y1;
-  origin.z = 0.f;
-  clutter_paint_volume_set_origin (&volume, &origin);
-  clutter_paint_volume_set_width (&volume, box->x2 - box->x1);
-  clutter_paint_volume_set_height (&volume, box->y2 - box->y1);
-
-  clutter_paint_volume_union (pv, &volume);
-
-  clutter_paint_volume_free (&volume);
-}
-
 /* The paint_volume setters only update vertices 0, 1, 3 and
  * 4 since the others can be drived from them.
  *
@@ -687,48 +549,33 @@ clutter_paint_volume_union_box (ClutterPaintVolume    *pv,
 void
 _clutter_paint_volume_complete (ClutterPaintVolume *pv)
 {
-  float dx_l2r, dy_l2r, dz_l2r;
-  float dx_t2b, dy_t2b, dz_t2b;
-
-  if (pv->is_empty)
+  if (pv->is_complete || pv->is_empty)
     return;
 
-  if (pv->is_complete)
-    return;
-
-  /* Find the vector that takes us from any vertex on the left face to
-   * the corresponding vertex on the right face. */
-  dx_l2r = pv->vertices[1].x - pv->vertices[0].x;
-  dy_l2r = pv->vertices[1].y - pv->vertices[0].y;
-  dz_l2r = pv->vertices[1].z - pv->vertices[0].z;
-
-  /* Find the vector that takes us from any vertex on the top face to
-   * the corresponding vertex on the bottom face. */
-  dx_t2b = pv->vertices[3].x - pv->vertices[0].x;
-  dy_t2b = pv->vertices[3].y - pv->vertices[0].y;
-  dz_t2b = pv->vertices[3].z - pv->vertices[0].z;
+  /* TODO: it is possible to complete non axis aligned volumes too. */
+  g_return_if_fail (pv->is_axis_aligned);
 
   /* front-bottom-right */
-  pv->vertices[2].x = pv->vertices[3].x + dx_l2r;
-  pv->vertices[2].y = pv->vertices[3].y + dy_l2r;
-  pv->vertices[2].z = pv->vertices[3].z + dz_l2r;
+  pv->vertices[2].x = pv->vertices[1].x;
+  pv->vertices[2].y = pv->vertices[3].y;
+  pv->vertices[2].z = pv->vertices[0].z;
 
   if (G_UNLIKELY (!pv->is_2d))
     {
       /* back-top-right */
-      pv->vertices[5].x = pv->vertices[4].x + dx_l2r;
-      pv->vertices[5].y = pv->vertices[4].y + dy_l2r;
-      pv->vertices[5].z = pv->vertices[4].z + dz_l2r;
+      pv->vertices[5].x = pv->vertices[1].x;
+      pv->vertices[5].y = pv->vertices[0].y;
+      pv->vertices[5].z = pv->vertices[4].z;
 
       /* back-bottom-right */
-      pv->vertices[6].x = pv->vertices[5].x + dx_t2b;
-      pv->vertices[6].y = pv->vertices[5].y + dy_t2b;
-      pv->vertices[6].z = pv->vertices[5].z + dz_t2b;
+      pv->vertices[6].x = pv->vertices[1].x;
+      pv->vertices[6].y = pv->vertices[3].y;
+      pv->vertices[6].z = pv->vertices[4].z;
 
       /* back-bottom-left */
-      pv->vertices[7].x = pv->vertices[4].x + dx_t2b;
-      pv->vertices[7].y = pv->vertices[4].y + dy_t2b;
-      pv->vertices[7].z = pv->vertices[4].z + dz_t2b;
+      pv->vertices[7].x = pv->vertices[0].x;
+      pv->vertices[7].y = pv->vertices[3].y;
+      pv->vertices[7].z = pv->vertices[4].z;
     }
 
   pv->is_complete = TRUE;
@@ -737,7 +584,7 @@ _clutter_paint_volume_complete (ClutterPaintVolume *pv)
 /*<private>
  * _clutter_paint_volume_get_box:
  * @pv: a #ClutterPaintVolume
- * @box: a pixel aligned #ClutterActorBox
+ * @box: a pixel aligned #ClutterGeometry
  *
  * Transforms a 3D paint volume into a 2D bounding box in the
  * same coordinate space as the 3D paint volume.
@@ -746,10 +593,9 @@ _clutter_paint_volume_complete (ClutterPaintVolume *pv)
  * the paint volume into window coordinates before getting
  * the 2D bounding box.
  *
- * The coordinates of the returned box are not clamped to
- * integer pixel values; if you need them to be rounded to the
- * nearest integer pixel values, you can use the
- * clutter_actor_box_clamp_to_pixel() function.
+ * <note>The coordinates of the returned box are not clamped to
+ * integer pixel values, if you need them to be clamped you can use
+ * clutter_actor_box_clamp_to_pixel()</note>
  *
  * Since: 1.6
  */
@@ -907,20 +753,19 @@ _clutter_paint_volume_axis_align (ClutterPaintVolume *pv)
   if (pv->is_empty)
     return;
 
+  g_return_if_fail (pv->is_complete);
+
   if (G_LIKELY (pv->is_axis_aligned))
     return;
 
   if (G_LIKELY (pv->vertices[0].x == pv->vertices[1].x &&
                 pv->vertices[0].y == pv->vertices[3].y &&
-                pv->vertices[0].z == pv->vertices[4].z))
+                pv->vertices[0].z == pv->vertices[4].y))
     {
       pv->is_axis_aligned = TRUE;
       return;
     }
 
-  if (!pv->is_complete)
-    _clutter_paint_volume_complete (pv);
-
   origin = pv->vertices[0];
   max_x = pv->vertices[0].x;
   max_y = pv->vertices[0].y;
@@ -993,7 +838,7 @@ _clutter_actor_set_default_paint_volume (ClutterActor       *self,
                                          GType               check_gtype,
                                          ClutterPaintVolume *volume)
 {
-  ClutterActorBox box;
+  ClutterGeometry geometry = { 0, };
 
   if (check_gtype != G_TYPE_INVALID)
     {
@@ -1008,14 +853,14 @@ _clutter_actor_set_default_paint_volume (ClutterActor       *self,
   if (!clutter_actor_has_allocation (self))
     return FALSE;
 
-  clutter_actor_get_allocation_box (self, &box);
+  clutter_actor_get_allocation_geometry (self, &geometry);
 
-  /* we only set the width and height, as the paint volume is defined
-   * to be relative to the actor's modelview, which means that the
-   * allocation's origin has already been applied
-   */
-  clutter_paint_volume_set_width (volume, box.x2 - box.x1);
-  clutter_paint_volume_set_height (volume, box.y2 - box.y1);
+  /* a zero-sized actor has no paint volume */
+  if (geometry.width == 0 || geometry.height == 0)
+    return FALSE;
+
+  clutter_paint_volume_set_width (volume, geometry.width);
+  clutter_paint_volume_set_height (volume, geometry.height);
 
   return TRUE;
 }
@@ -1028,8 +873,8 @@ _clutter_actor_set_default_paint_volume (ClutterActor       *self,
  * Sets the #ClutterPaintVolume from the allocation of @actor.
  *
  * This function should be used when overriding the
- * #ClutterActorClass.get_paint_volume() by #ClutterActor sub-classes
- * that do not paint outside their allocation.
+ * <function>get_paint_volume()</function> by #ClutterActor sub-classes that do
+ * not paint outside their allocation.
  *
  * A typical example is:
  *
@@ -1107,13 +952,12 @@ _clutter_paint_volume_cull (ClutterPaintVolume *pv,
           /* XXX: for perspective projections this can be optimized
            * out because all the planes should pass through the origin
            * so (0,0,0) is a valid v0. */
-          p.x = vertices[j].x - planes[i].v0[0];
-          p.y = vertices[j].y - planes[i].v0[1];
-          p.z = vertices[j].z - planes[i].v0[2];
+          p.x = vertices[j].x - planes[i].v0.x;
+          p.y = vertices[j].y - planes[i].v0.y;
+          p.z = vertices[j].z - planes[i].v0.z;
 
-          distance = (planes[i].n[0] * p.x +
-                      planes[i].n[1] * p.y +
-                      planes[i].n[2] * p.z);
+          distance =
+            planes[i].n.x * p.x + planes[i].n.y * p.y + planes[i].n.z * p.z;
 
           if (distance < 0)
             out++;
@@ -1140,17 +984,24 @@ _clutter_paint_volume_get_stage_paint_box (ClutterPaintVolume *pv,
   CoglMatrix modelview;
   CoglMatrix projection;
   float viewport[4];
-  float width;
-  float height;
 
   _clutter_paint_volume_copy_static (pv, &projected_pv);
 
+  /* NB: _clutter_actor_apply_modelview_transform_recursive will never
+   * include the transformation between stage coordinates and OpenGL
+   * eye coordinates, we have to explicitly use the
+   * stage->apply_transform to get that... */
   cogl_matrix_init_identity (&modelview);
 
   /* If the paint volume isn't already in eye coordinates... */
   if (pv->actor)
-    _clutter_actor_apply_relative_transformation_matrix (pv->actor, NULL,
-                                                         &modelview);
+    {
+      ClutterActor *stage_actor = CLUTTER_ACTOR (stage);
+      _clutter_actor_apply_modelview_transform (stage_actor, &modelview);
+      _clutter_actor_apply_modelview_transform_recursive (pv->actor,
+                                                          stage_actor,
+                                                          &modelview);
+    }
 
   _clutter_stage_get_projection_matrix (stage, &projection);
   _clutter_stage_get_viewport (stage,
@@ -1165,51 +1016,7 @@ _clutter_paint_volume_get_stage_paint_box (ClutterPaintVolume *pv,
                                  viewport);
 
   _clutter_paint_volume_get_bounding_box (&projected_pv, box);
-
-  /* The aim here is that for a given rectangle defined with floating point
-   * coordinates we want to determine a stable quantized size in pixels
-   * that doesn't vary due to the original box's sub-pixel position.
-   *
-   * The reason this is important is because effects will use this
-   * API to determine the size of offscreen framebuffers and so for
-   * a fixed-size object that may be animated accross the screen we
-   * want to make sure that the stage paint-box has an equally stable
-   * size so that effects aren't made to continuously re-allocate
-   * a corresponding fbo.
-   *
-   * The other thing we consider is that the calculation of this box is
-   * subject to floating point precision issues that might be slightly
-   * different to the precision issues involved with actually painting the
-   * actor, which might result in painting slightly leaking outside the
-   * user's calculated paint-volume. For this we simply aim to pad out the
-   * paint-volume by at least half a pixel all the way around.
-   */
-  width = box->x2 - box->x1;
-  height = box->y2 - box->y1;
-  width = CLUTTER_NEARBYINT (width);
-  height = CLUTTER_NEARBYINT (height);
-  /* XXX: NB the width/height may now be up to 0.5px too small so we
-   * must also pad by 0.25px all around to account for this. In total we
-   * must padd by at least 0.75px around all sides. */
-
-  /* XXX: The furthest that we can overshoot the bottom right corner by
-   * here is 1.75px in total if you consider that the 0.75 padding could
-   * just cross an integer boundary and so ceil will effectively add 1.
-   */
-  box->x2 = ceilf (box->x2 + 0.75);
-  box->y2 = ceilf (box->y2 + 0.75);
-
-  /* Now we redefine the top-left relative to the bottom right based on the
-   * rounded width/height determined above + a constant so that the overall
-   * size of the box will be stable and not dependant on the box's
-   * position.
-   *
-   * Adding 3px to the width/height will ensure we cover the maximum of
-   * 1.75px padding on the bottom/right and still ensure we have > 0.75px
-   * padding on the top/left.
-   */
-  box->x1 = box->x2 - width - 3;
-  box->y1 = box->y2 - height - 3;
+  clutter_actor_box_clamp_to_pixel (box);
 
   clutter_paint_volume_free (&projected_pv);
 }
@@ -1228,8 +1035,27 @@ _clutter_paint_volume_transform_relative (ClutterPaintVolume *pv,
   _clutter_paint_volume_set_reference_actor (pv, relative_to_ancestor);
 
   cogl_matrix_init_identity (&matrix);
-  _clutter_actor_apply_relative_transformation_matrix (actor,
-                                                       relative_to_ancestor,
+
+  if (relative_to_ancestor == NULL)
+    {
+      /* NB: _clutter_actor_apply_modelview_transform_recursive will never
+       * include the transformation between stage coordinates and OpenGL
+       * eye coordinates, we have to explicitly use the
+       * stage->apply_transform to get that... */
+      ClutterActor *stage = _clutter_actor_get_stage_internal (actor);
+
+      /* We really can't do anything meaningful in this case so don't try
+       * to do any transform */
+      if (G_UNLIKELY (stage == NULL))
+        return;
+
+      _clutter_actor_apply_modelview_transform (stage, &matrix);
+
+      relative_to_ancestor = stage;
+    }
+
+  _clutter_actor_apply_modelview_transform_recursive (actor,
+                                                      relative_to_ancestor,
                                                       &matrix);
 
   _clutter_paint_volume_transform (pv, &matrix);
