@@ -37,43 +37,36 @@
  * <refsect2 id="ClutterShaderEffect-implementing">
  *   <title>Implementing a ClutterShaderEffect</title>
  *   <para>Creating a sub-class of #ClutterShaderEffect requires the
- *   overriding of the #ClutterOffscreenEffectClass.paint_target() virtual
- *   function from the #ClutterOffscreenEffect class as well as the
- *   <function>get_static_shader_source()</function> virtual from the
- *   #ClutterShaderEffect class.</para>
- *   <para>The #ClutterShaderEffectClass.get_static_shader_source()
- *   function should return a copy of the shader source to use. This
- *   function is only called once per subclass of #ClutterShaderEffect
- *   regardless of how many instances of the effect are created. The
- *   source for the shader is typically stored in a static const
- *   string which is returned from this function via
- *   g_strdup().</para>
- *   <para>The <function>paint_target()</function> should set the
- *   shader's uniforms if any. This is done by calling
- *   clutter_shader_effect_set_uniform_value() or
- *   clutter_shader_effect_set_uniform(). The sub-class should then
- *   chain up to the #ClutterShaderEffect implementation.</para>
+ *   overriding of the <function>pre_paint()</function> virtual function
+ *   from the #ClutterEffect class.</para>
+ *   <para>The <function>pre_paint()</function> should set the shader's
+ *   source and eventually set the uniforms. The sub-class should call
+ *   clutter_shader_effect_set_shader_source() to set the shader source
+ *   code, and  clutter_shader_effect_set_uniform_value() or
+ *   clutter_shader_effect_set_uniform() to set the values of the shader
+ *   uniforms, if any; the sub-class should then chain up to the
+ *   #ClutterShaderEffect implementation.</para>
  *   <example id="ClutterShaderEffect-example-uniforms">
  *     <title>Setting uniforms on a ClutterShaderEffect</title>
  *     <para>The example below shows a typical implementation of the
- *     <function>get_static_shader_source()</function> and
- *     <function>paint_target()</function> phases of a
- *     #ClutterShaderEffect sub-class.</para>
+ *     <function>pre_paint()</function> phase of a #ClutterShaderEffect
+ *     sub-class.</para>
  *     <programlisting>
- *  static gchar *
- *  my_effect_get_static_shader_source (ClutterShaderEffect *effect)
- *  {
- *    return g_strdup (shader_source);
- *  }
- *
  *  static gboolean
- *  my_effect_paint_target (ClutterOffscreenEffect *effect)
+ *  my_effect_pre_paint (ClutterEffect *effect)
  *  {
  *    MyEffect *self = MY_EFFECT (effect);
  *    ClutterShaderEffect *shader = CLUTTER_SHADER_EFFECT (effect);
  *    ClutterEffectClass *parent_class;
  *    gfloat component_r, component_g, component_b;
  *
+ *    /&ast; if the effect is not enabled we can bail out now &ast;/
+ *    if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (effect)))
+ *      return FALSE;
+ *
+ *    /&ast; this function is a no-op after the first call &ast;/
+ *    clutter_shader_effect_set_shader_source (shader, shader_source);
+ *
  *    /&ast; the "tex" uniform is declared in the shader as:
  *     &ast;
  *     &ast;   uniform int tex;
@@ -99,8 +92,8 @@
  *                                       component_b);
  *
  *    /&ast; chain up to the parent's implementation &ast;/
- *    parent_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (my_effect_parent_class);
- *    return parent_class->paint_target (effect);
+ *    parent_class = CLUTTER_EFFECT_CLASS (my_effect_parent_class);
+ *    return parent_class->pre_paint (effect);
  *  }
  *     </programlisting>
  *   </example>
@@ -113,14 +106,10 @@
 #include "config.h"
 #endif
 
-/* XXX: This file depends on the cogl_program_ api with has been
- * removed for Cogl 2.0 so we undef COGL_ENABLE_EXPERIMENTAL_2_0_API
- * for this file for now */
-#undef COGL_ENABLE_EXPERIMENTAL_2_0_API
-#include "cogl/cogl.h"
-
 #include "clutter-shader-effect.h"
 
+#include "cogl/cogl.h"
+
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-feature.h"
@@ -132,7 +121,7 @@ typedef struct _ShaderUniform
   gchar *name;
   GType type;
   GValue value;
-  int location;
+  GLint location;
 } ShaderUniform;
 
 struct _ClutterShaderEffectPrivate
@@ -145,17 +134,10 @@ struct _ClutterShaderEffectPrivate
   CoglHandle shader;
 
   GHashTable *uniforms;
-};
 
-typedef struct _ClutterShaderEffectClassPrivate
-{
-  /* These are the per-class pre-compiled shader and program which is
-     used when the class implements get_static_shader_source without
-     calling set_shader_source. They will be shared by all instances
-     of this class */
-  CoglHandle program;
-  CoglHandle shader;
-} ClutterShaderEffectClassPrivate;
+  guint is_compiled : 1;
+  guint source_set  : 1;
+};
 
 enum
 {
@@ -168,12 +150,9 @@ enum
 
 static GParamSpec *obj_props[PROP_LAST];
 
-G_DEFINE_TYPE_WITH_CODE (ClutterShaderEffect,
-                         clutter_shader_effect,
-                         CLUTTER_TYPE_OFFSCREEN_EFFECT,
-                         G_ADD_PRIVATE (ClutterShaderEffect)
-                         g_type_add_class_private (g_define_type_id,
-                                                   sizeof (ClutterShaderEffectClassPrivate)))
+G_DEFINE_TYPE (ClutterShaderEffect,
+               clutter_shader_effect,
+               CLUTTER_TYPE_OFFSCREEN_EFFECT);
 
 static inline void
 clutter_shader_effect_clear (ClutterShaderEffect *self,
@@ -181,18 +160,15 @@ clutter_shader_effect_clear (ClutterShaderEffect *self,
 {
   ClutterShaderEffectPrivate *priv = self->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->shader);
-
-      priv->shader = COGL_INVALID_HANDLE;
-    }
+  if (priv->shader != COGL_INVALID_HANDLE && !priv->is_compiled)
+    cogl_handle_unref (priv->shader);
 
   if (priv->program != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->program);
 
       priv->program = COGL_INVALID_HANDLE;
+      priv->shader = COGL_INVALID_HANDLE;
     }
 
   if (reset_uniforms && priv->uniforms != NULL)
@@ -202,6 +178,8 @@ clutter_shader_effect_clear (ClutterShaderEffect *self,
     }
 
   priv->actor = NULL;
+  priv->is_compiled = FALSE;
+  priv->source_set = FALSE;
 }
 
 static void
@@ -230,7 +208,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
 
       if (CLUTTER_VALUE_HOLDS_SHADER_FLOAT (&uniform->value))
         {
-          const float *floats;
+          const GLfloat *floats;
 
           floats = clutter_value_get_shader_float (&uniform->value, &size);
           cogl_program_set_uniform_float (priv->program, uniform->location,
@@ -239,7 +217,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_INT (&uniform->value))
         {
-          const int *ints;
+          const GLint *ints;
 
           ints = clutter_value_get_shader_int (&uniform->value, &size);
           cogl_program_set_uniform_int (priv->program, uniform->location,
@@ -248,7 +226,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_MATRIX (&uniform->value))
         {
-          const float *matrix;
+          const GLfloat *matrix;
 
           matrix = clutter_value_get_shader_matrix (&uniform->value, &size);
           cogl_program_set_uniform_matrix (priv->program, uniform->location,
@@ -258,7 +236,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
         }
       else if (G_VALUE_HOLDS_FLOAT (&uniform->value))
         {
-          const float float_val = g_value_get_float (&uniform->value);
+          const GLfloat float_val = g_value_get_float (&uniform->value);
 
           cogl_program_set_uniform_float (priv->program, uniform->location,
                                           1, 1,
@@ -266,8 +244,8 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
         }
       else if (G_VALUE_HOLDS_DOUBLE (&uniform->value))
         {
-          const float float_val =
-            (float) g_value_get_double (&uniform->value);
+          const GLfloat float_val =
+            (GLfloat) g_value_get_double (&uniform->value);
 
           cogl_program_set_uniform_float (priv->program, uniform->location,
                                           1, 1,
@@ -275,7 +253,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
         }
       else if (G_VALUE_HOLDS_INT (&uniform->value))
         {
-          const int int_val = g_value_get_int (&uniform->value);
+          const GLint int_val = g_value_get_int (&uniform->value);
 
           cogl_program_set_uniform_int (priv->program, uniform->location,
                                         1, 1,
@@ -320,102 +298,52 @@ clutter_shader_effect_set_actor (ClutterActorMeta *meta,
                 G_OBJECT_TYPE_NAME (meta));
 }
 
-static CoglHandle
-clutter_shader_effect_create_shader (ClutterShaderEffect *self)
-{
-  ClutterShaderEffectPrivate *priv = self->priv;
-
-  switch (priv->shader_type)
-    {
-    case CLUTTER_FRAGMENT_SHADER:
-      return cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
-      break;
-
-    case CLUTTER_VERTEX_SHADER:
-      return cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
-      break;
-
-    default:
-      g_assert_not_reached ();
-    }
-}
-
-static void
-clutter_shader_effect_try_static_source (ClutterShaderEffect *self)
-{
-  ClutterShaderEffectPrivate *priv = self->priv;
-  ClutterShaderEffectClass *shader_effect_class =
-    CLUTTER_SHADER_EFFECT_GET_CLASS (self);
-
-  if (shader_effect_class->get_static_shader_source != NULL)
-    {
-      ClutterShaderEffectClassPrivate *class_priv;
-
-      class_priv =
-        G_TYPE_CLASS_GET_PRIVATE (shader_effect_class,
-                                  CLUTTER_TYPE_SHADER_EFFECT,
-                                  ClutterShaderEffectClassPrivate);
-
-      if (class_priv->shader == COGL_INVALID_HANDLE)
-        {
-          gchar *source;
-
-          class_priv->shader = clutter_shader_effect_create_shader (self);
-
-          source = shader_effect_class->get_static_shader_source (self);
-
-          cogl_shader_source (class_priv->shader, source);
-
-          g_free (source);
-
-          CLUTTER_NOTE (SHADER, "Compiling shader effect");
-
-          cogl_shader_compile (class_priv->shader);
-
-          if (cogl_shader_is_compiled (class_priv->shader))
-            {
-              class_priv->program = cogl_create_program ();
-
-              cogl_program_attach_shader (class_priv->program,
-                                          class_priv->shader);
-
-              cogl_program_link (class_priv->program);
-            }
-          else
-            {
-              gchar *log_buf = cogl_shader_get_info_log (class_priv->shader);
-
-              g_warning (G_STRLOC ": Unable to compile the GLSL shader: %s", log_buf);
-              g_free (log_buf);
-            }
-        }
-
-      priv->shader = cogl_handle_ref (class_priv->shader);
-
-      if (class_priv->program != COGL_INVALID_HANDLE)
-        priv->program = cogl_handle_ref (class_priv->program);
-    }
-}
-
 static void
 clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 {
-  ClutterShaderEffect *self = CLUTTER_SHADER_EFFECT (effect);
-  ClutterShaderEffectPrivate *priv = self->priv;
+  ClutterShaderEffectPrivate *priv = CLUTTER_SHADER_EFFECT (effect)->priv;
   ClutterOffscreenEffectClass *parent;
   CoglHandle material;
 
-  /* If the source hasn't been set then we'll try to get it from the
-     static source instead */
-  if (priv->shader == COGL_INVALID_HANDLE)
-    clutter_shader_effect_try_static_source (self);
-
   /* we haven't been prepared or we don't have support for
    * GLSL shaders in Clutter
    */
-  if (priv->program == COGL_INVALID_HANDLE)
+  if (priv->program == COGL_INVALID_HANDLE ||
+      priv->shader == COGL_INVALID_HANDLE)
     goto out;
 
+  if (!priv->source_set)
+    goto out;
+
+  if (!priv->is_compiled)
+    {
+      CLUTTER_NOTE (SHADER, "Compiling shader effect");
+
+      cogl_shader_compile (priv->shader);
+      if (!cogl_shader_is_compiled (priv->shader))
+        {
+          gchar *log_buf = cogl_shader_get_info_log (priv->shader);
+
+          g_warning ("Unable to compile the GLSL shader: %s", log_buf);
+          g_free (log_buf);
+
+          cogl_handle_unref (priv->shader);
+          priv->shader = COGL_INVALID_HANDLE;
+
+          cogl_handle_unref (priv->program);
+          priv->shader = COGL_INVALID_HANDLE;
+
+          goto out;
+        }
+
+      cogl_program_attach_shader (priv->program, priv->shader);
+      cogl_handle_unref (priv->shader);
+
+      cogl_program_link (priv->program);
+
+      priv->is_compiled = TRUE;
+    }
+
   CLUTTER_NOTE (SHADER, "Applying the shader effect of type '%s'",
                 G_OBJECT_TYPE_NAME (effect));
 
@@ -423,7 +351,7 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 
   /* associate the program to the offscreen target material */
   material = clutter_offscreen_effect_get_target (effect);
-  cogl_pipeline_set_user_program (material, priv->program);
+  cogl_material_set_user_program (material, priv->program);
 
 out:
   /* paint the offscreen buffer */
@@ -471,6 +399,8 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
 
   offscreen_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (ClutterShaderEffectPrivate));
+
   /**
    * ClutterShaderEffect:shader-type:
    *
@@ -502,7 +432,9 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
 static void
 clutter_shader_effect_init (ClutterShaderEffect *effect)
 {
-  effect->priv = clutter_shader_effect_get_instance_private (effect);
+  effect->priv = G_TYPE_INSTANCE_GET_PRIVATE (effect,
+                                              CLUTTER_TYPE_SHADER_EFFECT,
+                                              ClutterShaderEffectPrivate);
 }
 
 /**
@@ -516,7 +448,7 @@ clutter_shader_effect_init (ClutterShaderEffect *effect)
  * The effect will be empty until clutter_shader_effect_set_shader_source()
  * is called.
  *
- * Return value: the newly created #ClutterShaderEffect.
+ * Return value: (transfer full): the newly created #ClutterShaderEffect.
  *   Use g_object_unref() when done.
  *
  * Since: 1.8
@@ -634,8 +566,14 @@ clutter_shader_effect_add_uniform (ClutterShaderEffect *effect,
   else
     shader_uniform_update (uniform, value);
 
+  if (priv->is_compiled)
+    {
+      uniform->location =
+        cogl_program_get_uniform_location (priv->program, uniform->name);
+    }
+
   if (priv->actor != NULL && !CLUTTER_ACTOR_IN_PAINT (priv->actor))
-    clutter_effect_queue_repaint (CLUTTER_EFFECT (effect));
+    clutter_actor_queue_redraw (priv->actor);
 }
 
 /**
@@ -676,7 +614,7 @@ clutter_shader_effect_set_uniform_valist (ClutterShaderEffect *effect,
                                           gsize                n_values,
                                           va_list             *args)
 {
-  GValue value = G_VALUE_INIT;
+  GValue value = { 0, };
 
   if (value_type == CLUTTER_TYPE_SHADER_INT)
     {
@@ -789,7 +727,7 @@ add_uniform:
  * @name: the name of the uniform to set
  * @gtype: the type of the uniform to set
  * @n_values: the number of values
- * @...: a list of values
+ * @Varargs: a list of values
  *
  * Sets a list of values as the payload for the uniform @name inside
  * the shader effect
@@ -894,32 +832,35 @@ clutter_shader_effect_set_shader_source (ClutterShaderEffect *effect,
 
   priv = effect->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
+  if (priv->source_set)
     return TRUE;
 
-  priv->shader = clutter_shader_effect_create_shader (effect);
+  if (priv->program == COGL_INVALID_HANDLE)
+    priv->program = cogl_create_program ();
+
+  if (priv->shader == COGL_INVALID_HANDLE)
+    {
+      switch (priv->shader_type)
+        {
+        case CLUTTER_FRAGMENT_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
+          break;
+
+        case CLUTTER_VERTEX_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
+          break;
+
+        default:
+          priv->shader = COGL_INVALID_HANDLE;
+          break;
+        }
+    }
+
+  g_assert (priv->shader != COGL_INVALID_HANDLE);
 
   cogl_shader_source (priv->shader, source);
 
-  CLUTTER_NOTE (SHADER, "Compiling shader effect");
-
-  cogl_shader_compile (priv->shader);
-
-  if (cogl_shader_is_compiled (priv->shader))
-    {
-      priv->program = cogl_create_program ();
-
-      cogl_program_attach_shader (priv->program, priv->shader);
-
-      cogl_program_link (priv->program);
-    }
-  else
-    {
-      gchar *log_buf = cogl_shader_get_info_log (priv->shader);
-
-      g_warning (G_STRLOC ": Unable to compile the GLSL shader: %s", log_buf);
-      g_free (log_buf);
-    }
+  priv->source_set = TRUE;
 
   return TRUE;
 }
